/* SigmaStar trade secret */
/*
* cmd_efusectrl.c - Sigmastar
*
* Copyright (C) 2020 Sigmastar Technology Corp.
*
* Author: nick.lin <nick.lin@sigmastar.com.tw>
*
* This software is licensed under the terms of the GNU General Public
* License version 2, as published by the Free Software Foundation, and
* may be copied, distributed, and modified under those terms.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
*/

#include <common.h>
#include <command.h>
#include <environment.h>
#include <malloc.h>
#include "asm/arch/mach/ms_types.h"
#include "asm/arch/mach/platform.h"
#include "asm/arch/mach/io.h"

#define CMD_ENABLED_SECURE_BOOT 0x0

#define CMD_OFFSET 24
#define OFFSET_ERR_CODE    0x0
#define OFFSET_DATA_SZ     0x2
#define OFFSET_DATA        0x4

#define DBUG_LOG_R_8(_off)             (*(volatile unsigned char*)(CONFIG_EFUSECTRL_LOG_ADDRESS+_off))
#define DBUG_LOG_R_16(_off)            (*(volatile unsigned short*)(CONFIG_EFUSECTRL_LOG_ADDRESS+_off))
#define DBUG_LOG_W_16(_off,_val)       {(*((volatile unsigned short*)(CONFIG_EFUSECTRL_LOG_ADDRESS+_off))) = (unsigned short)(_val); }

void efuse_ctrl_W(unsigned long  efuse_cmd,unsigned long  val)
{
    __asm__ __volatile__("stmfd sp!,{r0-r3,lr}\n"
                         "mov r0,%0\n"
                         "mov r1,%1\n"
                         "swi 0x1\n"
                         "ldmfd sp!,{r0-r3,pc}\n"
                                       :
                                       :"r"(efuse_cmd),"r"(val)
                                       :"memory");
}

void efuse_ctrl_R(unsigned long  efuse_cmd)
{
    __asm__ __volatile__("stmfd sp!,{r0-r3,lr}\n"
                         "mov r0,%0\n"
                         "swi 0x2\n"
                         "ldmfd sp!,{r0-r3,pc}\n"
                                       :
                                       :"r"(efuse_cmd)
                                       :"memory");
}

void efuse_show_results(unsigned long op)
{
    int err_code = DBUG_LOG_R_16(OFFSET_ERR_CODE);

    if (err_code) {
       printf("[ERR] efuse control failure, error code:%x\n",err_code);
       return;
    }

    if (op == 0x2) {
        int i,j,k;
        unsigned long size = DBUG_LOG_R_16(OFFSET_DATA_SZ);

        if (size<=0)
            return;

        printf("Results: \n\r");

        for (i=0;i<size;i+=16)
        {
            k = (size-i) >16? 16:(size-i);
            printf("0x%02x:: ",i);
            for (j=i;j<i+k;j++)
            {
                printf("0x%02x ",DBUG_LOG_R_8(j+OFFSET_DATA));
            }

            printf("\n\r");
        }

    }

}

static int gs_efuse_read( unsigned int cmd, unsigned int offset, unsigned int *buf )
{
    unsigned int i, err_code;
    unsigned int size;
    unsigned char *data;

    efuse_ctrl_R( ( cmd << CMD_OFFSET ) | offset );

    err_code = DBUG_LOG_R_16(OFFSET_ERR_CODE);
    if ( err_code ) {
       printf( "otp control failure, error code:%x\n", err_code );
       return -1;
    }

    size = DBUG_LOG_R_16(OFFSET_DATA_SZ);
    data = (unsigned char *)buf;

    for ( i = 0; i < size; i ++ )
        data[i] = DBUG_LOG_R_8(i + OFFSET_DATA);
    DBUG_LOG_W_16(OFFSET_DATA_SZ,0);

    return 0;
}

// Return 1 is enabled secure boot, otherwise disabled
int gs_enabled_secure_boot(void)
{
    unsigned int value = 0;
    int ret;
    if ( gs_efuse_read(CMD_ENABLED_SECURE_BOOT, 0, &value) == 0 ) {
        if ( value == 0x01 )
            return 1;
    }

    return 0;
}

int do_efuse(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    unsigned long  cmd = 0;
    unsigned long  off = 0;
    unsigned long  val = 0;

    if ((!strncmp(argv[1], "-w", strlen("-w"))))
    {
        if (argc < 5)
        {
            return CMD_RET_USAGE;
        }

        cmd = (u32)simple_strtoul(argv[2], NULL, 16);
        off = (u32)simple_strtoul(argv[3], NULL, 16);
        val = (u32)simple_strtoul(argv[4], NULL, 16);
        efuse_ctrl_W((cmd<<CMD_OFFSET|off),val);
        efuse_show_results(0x1);
    }
    else if ((!strncmp(argv[1], "-r", strlen("-r"))))
    {
        if (argc < 3)
        {
            return CMD_RET_USAGE;
        }

        cmd = (u32)simple_strtoul(argv[2], NULL, 16);
        off = 0;
        efuse_ctrl_R((cmd<<CMD_OFFSET|off));
        if(cmd != 0x3)
        {
            efuse_show_results(0x2);
        }
        DBUG_LOG_W_16(OFFSET_DATA_SZ,0);
    }
    return CMD_RET_SUCCESS;
}

U_BOOT_CMD(
    efusectrl, CONFIG_SYS_MAXARGS, 1,	do_efuse,
    "Used for efuse program/read.",
    "Usage: efusectrl [-r] [efuse command] for reading data from efuse\n\r"
    "             efusectrl [-w] [efuse command] [address offset] [data] for programing data to efuse\n\r"
);





