/*
 * (C) Copyright 2013 Grandstream Networks, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <stdlib.h>
#include <common.h>
#include <command.h>
#include <linux/ctype.h>
#include <asm/arch/factory.h>

static char* strtolower(char* str)
{
    static char low[64];
    int i;

    memset( low, 0, sizeof ( low ) );
    for ( i = 0; i < strnlen( str, 63 ); i++ )
        low[i] = tolower(str[i]);

    return low;
}

static char* strtoupper(char* str)
{
    static char up[64];
    int i;

    memset( up, 0, sizeof ( up ) );
    for ( i = 0; i < strnlen( str, 63 ); i++ )
        up[i] = toupper(str[i]);

    return up;
}

int do_load_cfg(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    hw_info_t* hw_info = hw_get_hw_feature();
    char *cfgbuf = NULL;
    int cfglen;
#if 0
    uint16_t* mac = hw_get_mac();
    char buf[18];
#endif

    if ( hw_info == NULL ) {
		printf("do_load_cfg: error -- factory hw_info not available\n");
        return -1;
	}

    setenv( "MODEL", strtolower( hw_info->modelstr ) );
    setenv( "PNSTR", strtoupper( hw_get_pn_str() ) );
    if ( hw_info->oem_id != 0 ) {
        //setenv( "GSMODEL", strtolower( hw_info->oem_base ) );
        setenv( "GSMODEL", hw_info->oem_base );

        if( !hw_info->oem_base || !hw_info->modelalias || !hw_get_pn_str() ) {
		    printf("do_load_cfg: error -- Unable to get factory information, %d\n", __LINE__);
            return -1;
        }

        /* oembase_alias_PN.cfg */
        cfglen = strlen( hw_info->oem_base ) + strlen( hw_info->modelalias )\
                 + strlen( hw_get_pn_str() ) + 7;
        cfgbuf = malloc( cfglen );
        snprintf( cfgbuf, cfglen, "%s_%s_%s.cfg", hw_info->oem_base,
                hw_info->modelalias, hw_get_pn_str() );
    }
    else {
        setenv( "GSMODEL", strtolower( hw_info->gs_model ) );
        /* gsmodel_PN.cfg */
        cfglen = strlen( hw_info->gs_model ) + strlen( hw_get_pn_str() ) + 6;
        cfgbuf = malloc( cfglen );
        snprintf( cfgbuf, cfglen, "%s_%s.cfg", strtolower(hw_info->gs_model), hw_get_pn_str() );
    }
    setenv( "CFGFILE", cfgbuf );
    setenv( "CPU", strtolower( hw_info->cpu ) );

    free( cfgbuf );
#if 0
    if (mac) {
        snprintf(buf, 18, "%02x:%02x:%02x:%02x:%02x:%02x",
                (unsigned int)mac[0] & 0xFF,
                (unsigned int)( mac[0] >> 8 ) & 0xFF,
                (unsigned int)mac[1] & 0xFF,
                (unsigned int)( mac[1] >> 8 ) & 0xFF,
                (unsigned int)mac[2] & 0xFF,
                (unsigned int)( mac[2] >> 8 ) & 0xFF
            );
        setenv( "ethaddr", buf );
    }
#endif
    return 0;
}

/**************************************************/
U_BOOT_CMD(
    load_cfg, 1, 1, do_load_cfg, "load config", "" 
);

