/*
* pioneer3.h- Sigmastar
*
* Copyright (c) [2019~2020] SigmaStar Technology.
*
*
* This software is licensed under the terms of the GNU General Public
* License version 2, as published by the Free Software Foundation, and
* may be copied, distributed, and modified under those terms.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License version 2 for more details.
*
*/
#ifndef __PIONEER3_H
#define __PIONEER3_H

/*------------------------------------------------------------------------------
    Constant
-------------------------------------------------------------------------------*/
#define CONFIG_SILENT_CONSOLE   1   /* enable silent startup */

/* This is columbus2 hardware */
#define CONFIG_ARCH_PIONEER3    1
#define CONFIG_ARMCORTEXA7      /* This is an ARM V7 CPU core */
#define CONFIG_SYS_L2CACHE_OFF      /* No L2 cache */
/*#define CONFIG_SYS_ARCH_TIMER   1*/
#define CONFIG_MS_PIU_TIMER   1
#define CONFIG_AUTOBOOT_KEYED
#define CONFIG_AUTOBOOT_DELAY_STR "\x1B" /* press ESC to interrupt BOOT */
/* Efuse Base Address */
#define CONFIG_EFUSECTRL_FUN_ADDRESS              0xA0000010
#define CONFIG_EFUSECTRL_LOG_ADDRESS              0xA000D000


#define CONFIG_DISPLAY_BOARDINFO    1

#define CONFIG_BOARD_LATE_INIT

#if CONFIG_VERSION_FPGA
#define CONFIG_SYS_HZ_CLOCK 24000000
#define CONFIG_UART_CLOCK   24000000
#define CONFIG_BAUDRATE     115200
#define CONFIG_PIUTIMER_CLOCK 24000000
#else
#define CONFIG_SYS_HZ_CLOCK 400000000
#define CONFIG_UART_CLOCK   172800000
#define CONFIG_BAUDRATE     115200
#define CONFIG_PIUTIMER_CLOCK 12000000
#endif

#define CONFIG_WDT_CLOCK    CONFIG_PIUTIMER_CLOCK

#define CONFIG_FAT_WRITE

/* define baud rate */
#define CONFIG_SYS_BAUDRATE_TABLE   { 9600, 19200, 38400, 57600, 115200 }

/*------------------------------------------------------------------------------
    Macro
-------------------------------------------------------------------------------*/

/* boot delay time */
#define CONFIG_BOOTDELAY    0
#define CONFIG_ZERO_BOOTDELAY_CHECK

/*
#define CONFIG_MS_ANDROID_RECOVERY  1
#define CONFIG_MS_DISPLAY   1
#define CONFIG_MS_SHOW_LOGO 1
#define CONFIG_MS_ISP       1
#define CONFIG_MS_PIUTIMER 1
*/

#define CONFIG_SKIP_LOWLEVEL_INIT
/*#define CONFIG_DISPLAY_BOARDINFO *//*for checkboard*/
/*
 * Size of malloc() pool
 */
#define CONFIG_SYS_MALLOC_LEN       (10 << 20)

/*
 * Miscellaneous configurable options
 */
#define CONFIG_SYS_LONGHELP                     /* undef to save memory     */
#define CONFIG_SYS_PROMPT       "SigmaStar # "  /* Monitor Command Prompt   */
#define CONFIG_SYS_CBSIZE       512             /* Console I/O Buffer Size  */

/* Print Buffer Size */
#define CONFIG_SYS_PBSIZE           (CONFIG_SYS_CBSIZE+sizeof(CONFIG_SYS_PROMPT)+16)
#define CONFIG_SYS_MAXARGS          64          /* max number of command args   */
#define CONFIG_SYS_BARGSIZE     CONFIG_SYS_CBSIZE   /* Boot Argument Buffer Size    */

/*
 * Stack sizes
 *
 * The stack sizes are set up in start.S using the settings below
 */
#define CONFIG_STACKSIZE        (4*1024*1024)  /* regular stack */

/*
 * Physical Memory Map
 */
#define CONFIG_NR_DRAM_BANKS    1   /* we have 1 bank of DRAM */
#define PHYS_SDRAM_1        0x20000000  /* SDRAM Bank #1 */
#define PHYS_SDRAM_1_SIZE   CONFIG_SS_RAM_SIZE  /* 16/64 MB */

/*Enable watchdog*/
/*#define CONFIG_HW_WATCHDOG 1*/
#ifdef CONFIG_HW_WATCHDOG
#define CONFIG_HW_WATCHDOG_TIMEOUT_S    60
#endif


#define CONFIG_SYS_MEMTEST_START    0x20000000  /* memtest works on */
#define CONFIG_SYS_MEMTEST_END      0x22000000  /* 0 ... 32 MB in DRAM  */

#define CONFIG_SYS_SDRAM_BASE   PHYS_SDRAM_1
#define CONFIG_SYS_INIT_SP_ADDR     (CONFIG_SYS_TEXT_BASE  - GENERATED_GBL_DATA_SIZE)

#define CONFIG_UBOOT_RAM_SIZE   CONFIG_SS_RAM_SIZE // let us to use only 64MB for uboot



/* RAM base address */
#define RAM_START_ADDR          0x20000000
#define IMI_START_ADDR          0xA0000000

/* RAM size */
#define RAM_SIZE                PHYS_SDRAM_1_SIZE
/* The address used to save tag list used when kernel is booting */
#define BOOT_PARAMS             (RAM_START_ADDR)
#define BOOT_PARAMS_LEN         0x2000

/* CFG load address */
#define CONFIG_SYS_LOAD_ADDR            (BOOT_PARAMS+BOOT_PARAMS_LEN+0x4000)

#define CONFIG_CMDLINE_TAG       1    /* enable passing of ATAGs */
#define CONFIG_SETUP_MEMORY_TAGS 1
#define CONFIG_INITRD_TAG        1

/* kernel starting address */
#define KERNEL_RAM_BASE         CFG_LOAD_ADDR

/* Which block used to save IPL, u-boot and kernel images */
#define IPL_NAND_BLOCK      0
#define UBOOT_NAND_BLOCK    1
#define KERNEL_NAND_BLOCK   2


#define CONFIG_CMDLINE_EDITING 1
#define CONFIG_AUTO_COMPLETE

/* boot time analysis*/
#define CONFIG_BOOT_TIME_ANALYSIS           0
#define CONFIG_BOOT_TIME_ANALYSIS_USE_RTC   0

#define CONFIG_SYS_NO_FLASH                1

#define ENV_PART_NAME                   "env"
#define ENV_REDUND_PART_NAME            "ENV1"

#ifdef CONFIG_MS_ISP_FLASH
#define CONFIG_CMD_SF
#define SPI_BASE_ADDR           0x14000000
#ifdef CONFIG_MS_SAVE_ENV_IN_ISP_FLASH
#define CONFIG_ENV_IS_IN_SPI_FLASH
#define CONFIG_ENV_SECT_SIZE    0x1000
#define CONFIG_ENV_SIZE         0x1000  /* Total Size of Environment Sector */
#ifdef CONFIG_MS_PARTITION
#define CONFIG_ENV_OFFSET       CONFIG_MSTAR_ENV_NOR_OFFSET
#define CONFIG_MSTAR_ENV_NOR_OFFSET ms_nor_env_offset
#elif CONFIG_MSTAR_ENV_OFFSET
#define CONFIG_ENV_OFFSET       CONFIG_MSTAR_ENV_NOR_OFFSET
#define CONFIG_MSTAR_ENV_NOR_OFFSET ms_nor_env_offset
#else
/* bottom 4KB of available space in Uboot */
/* 0x40000 reserved for UBoot, 0x40000 maximum storage size of uboot */
#define CONFIG_ENV_OFFSET       0x1F000
#endif
#define FLASH_ENV_OFFSET        0x20
#ifdef CONFIG_MS_NOR_ENV_BACKUP
#define CONFIG_ENV_OFFSET_REDUND CONFIG_MSTAR_ENV_NOR_REDUND_OFFSET
#define CONFIG_MSTAR_ENV_NOR_REDUND_OFFSET ms_nor_env_redund_offset
#endif
#endif

#define CONFIG_CMD_UBI
/* #define CONFIG_CMD_UBIFS */
#define CONFIG_UBI_MWRITE

#define MTDIDS_DEFAULT			"nor0=nor0"
#define MTDPARTS_DEFAULT \
        "mtdparts=nor0:" \
        "124k(loader)," \
        "4k(env)," \
        "256k(uboota)," \
        "256k(ubootb)," \
        "128k(factory)," \
        "128k(nvram)," \
        "8k(manifest)," \
        "5m(base)," \
        "7680k(prog)," \
        "64k(oem)," \
        "1m(misc)," \
        "-(data)"

#define CONFIG_EXTRA_ENV_SETTINGS                              \
        "mtdids=" MTDIDS_DEFAULT "\0"                          \
        "mtdparts=" MTDPARTS_DEFAULT "\0"                      \
        "partition=nor0,0\0"                                  \
        "loadaddr=0x21000000\0"                                \
        "kernel_addr_load=0x21900000\0"                        \
        "kernel_addr_r=0x21900200\0"                           \
        "fact_addr_r=0x21800000\0"                             \
        "fdt_addr_r=0x21808000\0"                              \
        "gxp_console=gxp_console=0\0"                          \
        "read_kernel=sf probe 0 && sf read ${kernel_addr_load} base\0" \
        "loaddtb=sf probe 0 && sf read ${fact_addr_r} factory 20000\0"  \
        "bootcmd=gsboot; run detected_bootcmd\0" \
        "set_bootargs=setenv bootargs LX_MEM=0x3FE0000 mma_heap=mma_heap_name0,miu=0,sz=0x500000 cma=2M ${gxp_console} ${mtdparts}\0" \
        "flashboot=run set_bootargs && run loaddtb && run read_kernel && bootz ${kernel_addr_r} - ${fdt_addr_r}\0" \
        "updatefact=load_cfg && tftp ${MODEL}_${PNSTR}${FACTVER}.cfg && sf probe 0 && sf erase factory && sf write ${loadaddr} factory ${filesize}\0" \
        "updateboot=load_cfg && tftp ${GSMODEL}bootdev.bin && sf probe 0 && sf erase uboota && sf erase ubootb && sf write ${loadaddr} uboota ${filesize} && sf write ${loadaddr} ubootb ${filesize}\0" \
        "updatebase=load_cfg && tftp ${GSMODEL}base.bin && sf probe 0 && sf erase base && sf write ${loadaddr} base ${filesize} \0" \
        "updateprog=load_cfg && tftp ${GSMODEL}prog.bin && sf probe 0 && sf erase prog && sf write ${loadaddr} prog ${filesize} \0" \
        "updaterec=load_cfg && tftp ${GSMODEL}recovery.bin && sf probe 0 && sf erase recovery && sf write ${loadaddr} recovery ${filesize} \0" \
        "updateoem=load_cfg && tftp ${GSMODEL}oem.bin && sf probe 0 && sf erase oem && sf write ${loadaddr} oem ${filesize} \0" \
        "updatemf=load_cfg && tftp ${GSMODEL}manifest.bin && sf probe 0 && sf erase manifest && sf write ${loadaddr} manifest ${filesize} \0" \
        "updatemisc=load_cfg && tftp ${GSMODEL}misc.bin && sf probe 0 && sf erase misc && sf write ${loadaddr} misc ${filesize} \0" \
        "clearenv=sf probe 0 && sf erase env\0"     \
        "clearnvram=sf probe 0 && sf erase nvram\0" \
        "cleardata=sf probe 0 && sf erase data\0"   \
        "up_tftpboot=run set_bootargs && load_cfg && tftp ${loadaddr} zImage.${GSMODEL} && run loaddtb && bootz ${loadaddr} - ${fdt_addr_r}\0" \
        ""

#endif /*CONFIG_MS_ISP_FLASH*/


/*
 * File system, NAND flash, SD/MMC
 */




/*
 * FLASH driver setup
 */

#if defined(CONFIG_MS_SDMMC) || defined(CONFIG_MS_USB)
#define CONFIG_CMD_FAT
#endif

#ifdef CONFIG_MS_SDMMC
#define CONFIG_MMC
#define CONFIG_CMD_MMC
#define CONFIG_GENERIC_MMC
#define CONFIG_DOS_PARTITION
#define CONFIG_MS_SDMMC_MAX_READ_BLOCKS 1024
#define CONFIG_MMC_FDISK
#ifdef CONFIG_MS_SAVE_ENV_IN_SDMMC
#define CONFIG_ENV_IS_IN_MMC            1
#define CONFIG_SYS_MMC_ENV_DEV          0
#define CONFIG_ENV_SIZE                 0x20000
#define CONFIG_ENV_OFFSET               0x4F000
#endif
#if defined(CONFIG_FIRMWAREFS_ON_MMC)
#define CONFIG_FIRMWAREFS_ON_SDMMC_FLASH    1
#define PARTITION_MISC_INDEX                "MiscPart"
#endif
#endif

#ifdef CONFIG_MS_EMMC
#ifndef CONFIG_MMC
#define CONFIG_MMC
#define CONFIG_CMD_MMC
#define CONFIG_GENERIC_MMC
#endif
#if defined(CONFIG_FIRMWAREFS_ON_MMC)
#define CONFIG_FIRMWAREFS_ON_EMMC_FLASH     1
#endif
#endif

#ifdef CONFIG_MS_SPINAND
#if defined(CONFIG_MS_SAVE_ENV_IN_NAND_FLASH)
#define CONFIG_ENV_IS_IN_NAND
#define CONFIG_ENV_OFFSET       0x004C0000
#if defined(CONFIG_MSTAR_ENV_NAND_OFFSET)
#undef CONFIG_MSTAR_ENV_NAND_OFFSET
#endif
#define CONFIG_ENV_RANGE        0x00080000
/*#define CONFIG_MSTAR_ENV_NAND_OFFSET 0x440000*/
#ifdef CONFIG_MS_NAND_ENV_BACKUP
#define CONFIG_ENV_SIZE         0x00001000
#define CONFIG_ENV_OFFSET_REDUND CONFIG_MSTAR_ENV_NAND_REDUND_OFFSET
#define CONFIG_MSTAR_ENV_NAND_REDUND_OFFSET ms_nand_env_redund_offset
#else
#define CONFIG_ENV_SIZE         0x00080000
#endif
#endif

#define CONFIG_CMD_SPINAND_CIS
#define CONFIG_CMD_UBI
/* #define CONFIG_CMD_UBIFS */
#define CONFIG_UBI_MWRITE
#define MTDIDS_DEFAULT			"nand0=nand0"    /* "nor0=physmap-flash.0,nand0=nand" */
/*	must be different from real partition to test NAND partition function */
/*  #define MTDPARTS_DEFAULT		"mtdparts=nand0:0xC0000@0x140000(NPT),-(UBI)" */
/*	#define MTDPARTS_DEFAULT    "mtdparts=nand0:0x60000@0x140000(IPL0),0x60000(IPL1),0x60000(IPL_CUST0),0x60000(IPL_CUST1),0xC0000(UBOOT0),0xC0000(UBOOT1),0x60000(ENV),0x340000(KERNEL),0x340000(RECOVERY),-(UBI)"*/

/*
 * Note:
 *		nand flash block size is 128k
 *		cis : 1 block size, total 10 blocks, have 9 backup
 *		ipl : 1 block size, total 3 blocks, have 2 backup
 *		ipl_cust : 1 block size, total 3 blocks, have 2 backup
 * */
#define MTDPARTS_DEFAULT \
        "mtdparts=nand0:" \
        "1280k(cis)," \
        "384k(ipl)," \
        "384k(ipl_cust)," \
        "1152k(uboota)," \
        "1152k(ubootb)," \
        "512k(factory)," \
        "512K(env)," \
        "2m(nvram)," \
        "50m(ubi0)," \
        "50m(ubi1)," \
        "-(data)"

#define CONFIG_EXTRA_ENV_SETTINGS                              \
        "mtdids=" MTDIDS_DEFAULT "\0"                          \
        "mtdparts=" MTDPARTS_DEFAULT "\0"                      \
        "partition=nand0,0\0"                                  \
        "loadaddr=0x21000000\0"                                \
        "kernel_addr_load=0x21900000\0"                        \
        "kernel_addr_r=0x21900200\0"                           \
        "fact_addr_r=0x21800000\0"                             \
        "fdt_addr_r=0x21808000\0"                              \
        "gxp_console=gxp_console=0\0"                          \
        "rootdevice=rootfsname=ubi0:base\0"                    \
        "memsize=128M\0"                                       \
        "rootfstype=squashfs\0"                                \
        "read_kernel=ubi part ubi0; ubi read ${kernel_addr_load} core\0" \
		"loaddtb=nand read ${fact_addr_r} factory 20000\0"  \
        "bootcmd=gsboot; run detected_bootcmd\0" \
        "set_bootargs=setenv bootargs ${gxp_console} gluebi=ubi0:ubi1 ubi.mtd=ubi0 ubi.mtd=ubi1 ${rootdevice} rootfstype=${rootfstype} mem=${memsize} ${mtdparts}\0" \
        "flashboot=run set_bootargs; run loaddtb; run read_kernel; bootz ${kernel_addr_r} - ${fdt_addr_r}\0" \
        "updateall=run updateloader updatefact updateboot updatefirmware clearenv clearnvram cleardata\0" \
        "updatefact=load_cfg; tftp ${MODEL}_${PNSTR}${FACTVER}.cfg; nand erase.part factory; nand write.e ${loadaddr} factory ${filesize}\0" \
        "updateboot=load_cfg; tftp ${GSMODEL}bootdev.bin; nand erase.part uboota; nand write.e ${loadaddr} uboota ${filesize}; nand erase.part ubootb; nand write.e ${loadaddr} ubootb ${filesize}\0" \
        "updatefirmware=load_cfg; tftp ${GSMODEL}firmware.bin; nand erase.part ubi0; nand erase.part ubi1; nand write.e ${loadaddr} ubi0 ${filesize}; nand write.e ${loadaddr} ubi1 ${filesize}\0" \
        "clearenv=nand erase.part env\0"     \
        "clearnvram=nand erase.part nvram\0" \
        "cleardata=nand erase.part data\0"   \
        "up_testboot=tftp boot.${GSMODEL}; nand erase.part uboota; nand erase.part ubootb; nand write.e ${loadaddr} uboota ${filesize};\0"   \
        "up_testfw=tftp ${GSMODEL}firmware.bin; nand erase.part ubi0; nand erase.part ubi1; nand write.e ${loadaddr} ubi0 ${filesize}\0"   \
        "up_tftpboot=run set_bootargs; load_cfg; tftp ${loadaddr} zImage.${GSMODEL}; run loaddtb; bootz ${loadaddr} - ${fdt_addr_r}\0" \

        ""

#define CONFIG_CMD_NAND

#ifdef CONFIG_CMD_NAND
#define CONFIG_SYS_NAND_MAX_CHIPS   1
#define CONFIG_SYS_MAX_NAND_DEVICE  1
#define CONFIG_SYS_NAND_BASE        0 /* not actually used */
#endif /*CONFIG_CMD_NAND*/

#endif /*CONFIG_MS_SPINAND*/

#ifdef CONFIG_MS_NAND

/*
#ifndef CONFIG_ENV_IS_IN_SPI_FLASH
#define CONFIG_ENV_IS_IN_NAND
#define CONFIG_ENV_OFFSET_OOB
#endif

*/

/* #define ENABLE_NAND_RIU_MODE 1 */

#define CONFIG_CMD_UBI
/* #define CONFIG_CMD_UBIFS */
#define CONFIG_UBI_MWRITE

#define MTDIDS_DEFAULT          "nand0=nand0"    /* "nor0=physmap-flash.0,nand0=nand" */
#define MTDPARTS_DEFAULT        "mtdparts=nand0:0xC0000@0x140000(NPT),-(UBI)"  /* must be different from real partition to test NAND partition function */

#define CONFIG_EXTRA_ENV_SETTINGS                              \
       "mtdids=" MTDIDS_DEFAULT "\0"                           \
       "mtdparts=" MTDPARTS_DEFAULT "\0"                       \
       "partition=nand0,0\0"                                   \
       ""

#define CONFIG_CMD_NAND
#define CONFIG_CMD_CIS

#ifdef CONFIG_CMD_NAND
#define CONFIG_SYS_NAND_MAX_CHIPS   1
#define CONFIG_SYS_MAX_NAND_DEVICE  1
#define CONFIG_SYS_NAND_BASE        0 /* not actually used */
#endif

#endif

#ifdef CONFIG_MS_EMMC
#ifdef CONFIG_MS_SAVE_ENV_IN_EMMC
#define CONFIG_ENV_IS_IN_MMC       1
#define CONFIG_SYS_MMC_ENV_DEV     0
#define CONFIG_MS_EMMC_DEV_INDEX   1
#define CONFIG_EMMC_PARTITION
#define CONFIG_UNLZO_DST_ADDR  0x24000000
#define CONFIG_ENV_SIZE         0x00020000
#define CONFIG_ENV_OFFSET          0x4F000
#else
#define CONFIG_MS_EMMC_DEV_INDEX   0
#define CONFIG_EMMC_PARTITION
#endif
#endif

#if defined(CONFIG_ENV_IS_IN_NAND) || defined(CONFIG_ENV_IS_IN_MMC) || defined(CONFIG_ENV_IS_IN_SPI_FLASH) || defined(CONFIG_MS_SPINAND)

#define CONFIG_CMD_SAVEENV  /* saveenv */
#else
#define CONFIG_ENV_IS_NOWHERE
#define CONFIG_ENV_SIZE         0x00020000  /* Total Size of Environment Sector */
#define CONFIG_ENV_RANGE        0x00060000

#endif

#if defined(CONFIG_MS_NAND) || defined(CONFIG_MS_MTD_ISP_FLASH) || defined(CONFIG_MS_SPINAND)
#define CONFIG_RBTREE
#define CONFIG_MTD_DEVICE               /* needed for mtdparts commands */
#define CONFIG_MTD_PARTITIONS
#define CONFIG_CMD_MTDPARTS
#endif


/* CMD */

#define CONFIG_CMD_CACHE
#define CONFIG_CMD_MEMORY   /* md mm nm mw cp cmp crc base loop mtest */
#define CONFIG_CMD_RUN      /* run command in env variable  */
#define CONFIG_AUTOSCRIPT

/*
 * USB configuration
 */
#ifdef CONFIG_MS_USB
#if !defined(CONFIG_USB)
    #define     CONFIG_USB
#endif
#define     CONFIG_CMD_USB
#if !defined(CONFIG_USB_STORAGE)
    #define     CONFIG_USB_STORAGE
#endif

#ifdef CONFIG_MS_ENABLE_USB_LAN_MODULE
#define ENABLE_USB_LAN_MODULE
#define CONFIG_CMD_NET
#define CONFIG_CMD_DHCP
#define CONFIG_CMD_PING
#define CONFIG_BOOTP_GATEWAY
#endif

#endif



/*following config should be considered to be removed after confirmed*/

/* #define CONFIG_SKIP_RELOCATE_UBOOT dropped after uboot 201012 */
/*
#undef CONFIG_USE_IRQ   // we don't need IRQ/FIQ stuff
*/
/*#define KERNEL_IMAGE_SIZE       0x1000000 // 10M  kernel image size */

/*  move to cedric_defconfig
 *
 *  #define CONFIG_MS_SDMMC     1
 *  #define CONFIG_MS_EMMC      1
 *  #define CONFIG_MS_NAND      1
 *
 */

 /* Ethernet configuration */
#ifdef CONFIG_MS_EMAC
#define CONFIG_MINIUBOOT
#define CONFIG_BOOTP_GATEWAY
#define CONFIG_TFTP_PORT
#endif

#define CONFIG_XZ
#define CONFIG_MZ

#define XZ_DICT_ADDR            0x21000000  /*used for XZ decompress*/
#define XZ_DICT_LENGTH          0x01000000

/* I2C configuration */
#define CONFIG_CMD_I2C
#define CONFIG_SYS_I2C
#define CONFIG_I2C_MULTI_BUS
#define CONFIG_SYS_I2C_MSTAR
#define I2C_MS_DECLARATIONS2
#define CONFIG_SYS_I2C_MS_SPEED (300000)
#define CONFIG_SYS_I2C_MS_SLAVE (0x7F)
/*
 * padmux: 1 -> PAD_I2C0_SCL, PAD_I2C0_SDA
 *         2 -> PAD_PWM0, PAD_PWM1
 *         3 -> PAD_SR_IO00, PAD_SR_IO01
 */
#define CONFIG_SYS_I2C_0_PADMUX (0xB)
/*
 * padmux: 1 -> PAD_I2C1_SCL, PAD_I2C1_SDA
 *         2 -> PAD_PWM0, PAD_PWM1
 *         3 -> PAD_SR_IO00, PAD_SR_IO01
 */
#define CONFIG_SYS_I2C_1_PADMUX (0x6)

/* EEPROM */
#define CONFIG_CMD_EEPROM
#define CONFIG_HARD_I2C
#define CONFIG_SYS_I2C_EEPROM_ADDR_LEN (2)
#define CONFIG_SYS_I2C_EEPROM_ADDR (0x54)

/*
#define ENABLE_DOUBLE_SYSTEM_CHECK  1
*/

/* SENSOR */
#define CONFIG_MS_SRCFG

#if defined(CONFIG_USB_GADGET)
    #ifdef CONFIG_ARMCORTEXA7
        #define CONFIG_SYS_CACHELINE_SIZE   (64)
    #endif
    #if defined(CONFIG_USBDOWNLOAD_GADGET)
        #define CONFIG_G_DNL_VENDOR_NUM         0x1D6B
        #define CONFIG_G_DNL_PRODUCT_NUM        0x0101
        #define CONFIG_G_DNL_MANUFACTURER       "SigmaStar"
        #define CONFIG_USB_GADGET_VBUS_DRAW     2

        #if defined(CONFIG_CMD_FASTBOOT)
            #define CONFIG_USB_FASTBOOT_BUF_ADDR    (BOOT_PARAMS + 0x1000000)
            #define CONFIG_USB_FASTBOOT_BUF_SIZE    0x500000
        #endif
    #endif
    #if defined(CONFIG_SSTAR_DNL_GADGET)
        #undef CONFIG_USB_GADGET_VBUS_DRAW
        #define CONFIG_USB_GADGET_VBUS_DRAW     0
        #define CONFIG_MD5
    #endif
    #define CONFIG_USB_GADGET_DUALSPEED
#endif

#endif  /* __PIONEER3_H */
