/*
* mhal_pinmux.c- Sigmastar
*
* Copyright (c) [2019~2020] SigmaStar Technology.
*
*
* This software is licensed under the terms of the GNU General Public
* License version 2, as published by the Free Software Foundation, and
* may be copied, distributed, and modified under those terms.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License version 2 for more details.
*
*/
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/delay.h>
#include <linux/kdev_t.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/poll.h>
#include <linux/wait.h>
#include <linux/cdev.h>
#include <linux/time.h>
#include <linux/timer.h>
#include <asm/io.h>
#include "ms_platform.h"
#include "mdrv_types.h"
#include "padmux.h"
#include "gpio.h"

#define PADTOP1_BASE_ADDR 0x1026 
#define PMSLEEP_BASE_ADDR 0x000E
#define PM_SAR_BASE_ADDR  0x0014
#define ALBANY2_BASE_ADDR 0x0033
#define UTMI0_BASE_ADDR   0x1421
#define UTMI1_BASE_ADDR   0x1429


//16bit offset
//PADTOP1
#define REG_ALL_PAD_IN                      0x00
#define REG_FORTHUART_MODE           0x01
#define REG_SECONDUART_MODE         0x01
#define REG_THIRDUART_MODE            0x01
#define REG_BT_I2S_RX_0_MODE          0x02
#define REG_BT_I2S_TRX_0_MODE        0x02
#define REG_BT_I2S_TX_0_MODE          0x02
#define REG_CCIR0_16B_MODE             0x02
#define REG_CCIR0_8B_MODE               0x02
#define REG_CCIR0_CLK_MODE             0x02
#define REG_CCIR0_CTRL_MODE           0x02
#define REG_CCIR1_8B_MODE               0x02
#define REG_CCIR1_CLK_MODE             0x02
#define REG_CCIR1_CTRL_MODE           0x02
#define REG_CCIR2_16B_MODE             0x02
#define REG_CCIR2_8B_MODE               0x02
#define REG_CCIR2_CLK_MODE             0x03
#define REG_CCIR2_CTRL_MODE           0x03
#define REG_CCIR3_8B_MODE               0x03
#define REG_CCIR3_CLK_MODE             0x03
#define REG_CCIR3_CTRL_MODE           0x03
#define REG_CODEC_I2S_RX_0_MODE   0x03
#define REG_CODEC_I2S_RX_1_MODE   0x03
#define REG_CODEC_I2S_RX_2_MODE   0x03
#define REG_CODEC_I2S_RX_3_MODE   0x03
#define REG_CODEC_I2S_RX_MCK_MODE   0x03
#define REG_CODEC_I2S_TX_0_MODE        0x03
#define REG_CODEC_I2S_TX_MUTE_MODE 0x03
#define REG_DMIC_0_MODE                 0x04
#define REG_DMIC_1_MODE                 0x04
#define REG_DMIC_2_MODE                 0x04
#define REG_DMIC_3_MODE                 0x04
#define REG_EJ_CEVA_MODE                0x04
#define REG_EJ_MODE                          0x04
#define REG_EMMC_CONFIG                  0x04
#define REG_EMMC_RSTN_EN               0x04
#define REG_FAST_UART_RTX_MODE    0x05
#define REG_FUART_EMMC_MODE         0x05
#define REG_FUART_MODE                    0x05
#define REG_GT0_MDIO                        0x05
#define REG_GT0_MODE                        0x05
#define REG_GT1_MODE                        0x05
#define REG_HDMIRX_ARC_MODE          0x05
#define REG_HDMITX_ARC_MODE          0x05
#define REG_HDMITX_DDC_MODE          0x05
#define REG_HSYNC_EN                         0x05
#define REG_I2CM0_MODE                     0x06
#define REG_I2CM1_MODE                     0x06
#define REG_I2CM2_MODE                     0x06
#define REG_I2CM3_MODE                     0x06
#define REG_I2CM4_MODE                     0x06
#define REG_I2S_IN_MODE                    0x06
#define REG_I2S_OUT_MODE                 0x06
#define REG_I2S_OUT_MODE2               0x07
#define REG_I2S_OUT_MUTE_MODE       0x07
#define REG_I2S_TRX_MODE                   0x07
#define REG_MIPI_LVDS_TX_2CH_MODE  0x07
#define REG_MIPI_LVDS_TX_4CH_MODE  0x07
#define REG_MISC_I2S_RX_0_MODE        0x07
#define REG_MISC_I2S_RX_MCK_MODE    0x07
#define REG_MISC_I2S_TX_0_MODE         0x07
#define REG_MISC_I2S_TX_1_MODE         0x07
#define REG_MISC_I2S_TX_2_MODE         0x07
#define REG_MSPI1_MODE1                      0x08
#define REG_MSPI1_MODE2                      0x08
#define REG_MSPI1_MODE3                      0x08
#define REG_MSPI2_MODE1                      0x08
#define REG_MSPI3_MODE1                      0x08
#define REG_NAND_CS1_EN                      0x08
#define REG_NAND_MODE                         0x08
#define REG_PWM0_MODE                        0x09
#define REG_PWM1_MODE                        0x09
#define REG_PWM2_MODE                        0x09
#define REG_PWM3_MODE                        0x09
#define REG_PWM4_MODE                        0x09
#define REG_PWM5_MODE                        0x09
#define REG_PWM6_MODE                        0x09
#define REG_PWM7_MODE                        0x09
#define REG_RGB_16B_MODE                    0x0a
#define REG_RGB_24B_MODE                    0x0a
#define REG_SATA1_LED_MODE                 0x0a
#define REG_SATA_LED_MODE                   0x0a
#define REG_SD_CONFIG                            0x0a
#define REG_SDIO30_MODE                       0x0a
#define REG_SDIO_MODE                           0x0a
#define REG_SEC_MSPI2_MODE                  0x0a
#define REG_SNR0_IN_MODE                     0x0a
#define REG_SNR1_IN_MODE                     0x0a
#define REG_SNR2_IN_MODE                     0x0a
#define REG_SNR3_IN_MODE                     0x0a
#define REG_SPDIF_IN_MODE                    0x0b
#define REG_SPDIF_OUT_MODE                 0x0b
#define REG_TEST_IN_MODE                     0x0b
#define REG_TEST_OUT_MODE                  0x0b
#define REG_TTL_OUT                               0x0b
#define REG_USB30VCTL_MODE                 0x0b
#define REG_USB30VCTL_MODE1               0x0b
#define REG_VSYNC_EN                      0x0b


//PMSLEEP
#define REG_GPIO_PM_LOCK    0x12
#define REG_IR_IS_GPIO      0x1c
#define REG_SD_CDZ_MODE     0x28
#define REG_LED_MODE        0x28
#define REG_VID_MODE        0x28
#define REG_GPU_VID_MODE        0x28
#define REG_MIIC_MODE        0x28
#define REG_SD_CDZ_MODE     0x28
#define REG_PM_PWM0_MODE     0x28
#define REG_PM_PWM1_MODE     0x28
#define REG_SPI_IS_GPIO     0x35
#define REG_MSPI_MODE     0x28
#define REG_HDMI_HPD_BYPASS 0x27

//SAR
#define REG_SAR_MODE        0x11

//EMAC ALBANY2_BASE_ADDR
//#define REG_ETH_GPIO_EN     0x71



typedef struct stPadmux
{
    U16 padID;
    U16 base;
    U16 offset;
    U16 mask;
    U16 val;
    U16 mode;
} ST_PADMUX;

ST_PADMUX padmux_table[]=
{
    /* GPIO0-3 - only config as GPIO mode */
//    {PAD_GPIO0, PADTOP1_BASE_ADDR, ?,       ?, ?,       PINMUX_FOR_GPIO_MODE}, /*   */
//    {PAD_GPIO1, PADTOP1_BASE_ADDR, ? ,      ?, ?,       PINMUX_FOR_GPIO_MODE}, /*   */
//    {PAD_GPIO2, PADTOP1_BASE_ADDR, ?,       ?, ?,       PINMUX_FOR_GPIO_MODE}, /*   */
//    {PAD_GPIO3, PADTOP1_BASE_ADDR, ?,       ?, ?,       PINMUX_FOR_GPIO_MODE}, /*   */

    {PAD_GPIO4, PADTOP1_BASE_ADDR, REG_I2CM2_MODE,      BIT7|BIT6|BIT5,      BIT6|BIT5,  PINMUX_FOR_I2CM2_MODE}, /* i2cm2 mode 3 */
    {PAD_GPIO5, PADTOP1_BASE_ADDR, REG_I2CM2_MODE,      BIT7|BIT6|BIT5,      BIT6|BIT5,  PINMUX_FOR_I2CM2_MODE}, /* i2cm2 mode 3 */
    {PAD_GPIO6, PADTOP1_BASE_ADDR, REG_SATA_LED_MODE,      BIT5|BIT4,      BIT4,  PINMUX_FOR_SATA_LED_MODE}, /* sata_led mode 1 */
    {PAD_GPIO7, PADTOP1_BASE_ADDR, REG_SATA1_LED_MODE,      BIT3|BIT2,      BIT2,  PINMUX_FOR_SATA1_LED_MODE}, /* sata_led mode 1 */
    {PAD_GPIO8, PADTOP1_BASE_ADDR, REG_SPDIF_OUT_MODE,       BIT3|BIT2,      BIT3,       PINMUX_FOR_SPDIF_OUT_MODE}, /* SPDIF mode 2 */
    {PAD_GPIO8, PADTOP1_BASE_ADDR, REG_PWM2_MODE,        BIT5|BIT4,           BIT4,       PINMUX_FOR_PWM2_MODE}, /* pwm2 mode 1 */
    {PAD_GPIO9, PADTOP1_BASE_ADDR, REG_PWM3_MODE,       BIT7|BIT6,      BIT6,       PINMUX_FOR_PWM3_MODE}, /*pwm3 mode 1 */
    {PAD_GPIO10, PADTOP1_BASE_ADDR, REG_MSPI1_MODE1,      BIT1|BIT0, BIT1,       PINMUX_FOR_MSPI1_MODE1}, /* mspi1 mode 2*/
    {PAD_GPIO10, PADTOP1_BASE_ADDR, REG_PWM4_MODE,      BIT1|BIT0,  BIT0,       PINMUX_FOR_PWM4_MODE}, /* pwm4 mode 1 */
    {PAD_GPIO11, PADTOP1_BASE_ADDR, REG_PWM5_MODE,      BIT3|BIT2,  BIT2,       PINMUX_FOR_PWM5_MODE}, /* pwm5 mode 1 */
    {PAD_GPIO12, PADTOP1_BASE_ADDR, REG_PWM6_MODE,      BIT6|BIT5,  BIT5,       PINMUX_FOR_PWM6_MODE}, /* pwm6 mode 1 */
    {PAD_GPIO13, PADTOP1_BASE_ADDR, REG_PWM7_MODE,      BIT7|BIT6,  BIT6,       PINMUX_FOR_PWM7_MODE}, /* pwm7 mode 1 */
    {PAD_GPIO14, PADTOP1_BASE_ADDR, REG_USB30VCTL_MODE,      BIT2|BIT1,     BIT2|BIT1, PINMUX_FOR_USB30VCTL_MODE}, /* usb30vctl mode 2  */
    {PAD_GPIO14, PADTOP1_BASE_ADDR, REG_USB30VCTL_MODE1,      BIT4|BIT3,   BIT4|BIT3,       PINMUX_FOR_USB30VCTL_MODE1}, /* usb30vctl mode 3 */
    {PAD_GPIO15, PADTOP1_BASE_ADDR, REG_USB30VCTL_MODE,      BIT2|BIT1,     BIT2,       PINMUX_FOR_USB30VCTL_MODE}, /* usb30vctl mode 2 */

    {PAD_PM_GPIO0, PMSLEEP_BASE_ADDR, REG_SPI_IS_GPIO, BIT7|BIT6,      BIT6,       PINMUX_FOR_PMSPI_MODE},
    {PAD_PM_GPIO0, PMSLEEP_BASE_ADDR, REG_PM_PWM0_MODE, BIT1|BIT0,      BIT0,       PINMUX_FOR_PWM0_MODE},
    {PAD_PM_GPIO0, PMSLEEP_BASE_ADDR, REG_VID_MODE, BIT5|BIT4,      BIT5,       PINMUX_FOR_VID_MODE},
    {PAD_PM_GPIO0, PMSLEEP_BASE_ADDR, REG_GPU_VID_MODE, BIT3|BIT2,      BIT3,       PINMUX_FOR_GPU_VID_MODE},
    {PAD_PM_GPIO0, PMSLEEP_BASE_ADDR, REG_MIIC_MODE, BIT7|BIT6,      BIT6,       PINMUX_FOR_MIIC_MODE},
    {PAD_PM_GPIO1, PMSLEEP_BASE_ADDR, REG_SPI_IS_GPIO, BIT7|BIT6,      BIT7,       PINMUX_FOR_PMSPI_MODE},
    {PAD_PM_GPIO1, PMSLEEP_BASE_ADDR, REG_PM_PWM1_MODE, BIT3|BIT2,      BIT2,       PINMUX_FOR_PWM1_MODE},
//    {PAD_GPIO2, PADTOP1_BASE_ADDR, ?,       ?, ?,       PINMUX_FOR_GPIO_MODE}, /*   */
    {PAD_PM_GPIO3, PMSLEEP_BASE_ADDR, REG_VID_MODE, BIT5|BIT4,      BIT5,       PINMUX_FOR_VID_MODE},
    {PAD_PM_GPIO3, PMSLEEP_BASE_ADDR, REG_GPU_VID_MODE, BIT3|BIT2,      BIT3,       PINMUX_FOR_GPU_VID_MODE},
    {PAD_PM_GPIO3, PMSLEEP_BASE_ADDR, REG_MIIC_MODE, BIT7|BIT6,      BIT6,       PINMUX_FOR_MIIC_MODE},
    /* PAD_PM_GPIO4 special case */
//    {PAD_GPIO5, PADTOP1_BASE_ADDR, ?,       ?, ?,       PINMUX_FOR_GPIO_MODE}, /*   */
//    {PAD_GPIO6, PADTOP1_BASE_ADDR, ?,       ?, ?,       PINMUX_FOR_GPIO_MODE}, /*   */
//    {PAD_GPIO7, PADTOP1_BASE_ADDR, ?,       ?, ?,       PINMUX_FOR_GPIO_MODE}, /*   */
    {PAD_PM_GPIO8, PMSLEEP_BASE_ADDR, REG_SPI_IS_GPIO,  BIT3,      BIT3,       PINMUX_FOR_PMSPI_MODE},
//    {PAD_GPIO9, PADTOP1_BASE_ADDR, ?,       ?, ?,       PINMUX_FOR_GPIO_MODE}, /*   */
//    {PAD_GPIO10, PADTOP1_BASE_ADDR, ?,       ?, ?,       PINMUX_FOR_GPIO_MODE}, /*   */
//    {PAD_GPIO11, PADTOP1_BASE_ADDR, ?,       ?, ?,       PINMUX_FOR_GPIO_MODE}, /*   */
    {PAD_PM_GPIO12, PMSLEEP_BASE_ADDR, REG_MSPI_MODE, BIT15,  BIT15,       PINMUX_FOR_PM_MSPI_MODE},
    {PAD_PM_GPIO13, PMSLEEP_BASE_ADDR, REG_MSPI_MODE, BIT15,  BIT15,       PINMUX_FOR_PM_MSPI_MODE},
    {PAD_PM_GPIO14, PMSLEEP_BASE_ADDR, REG_MSPI_MODE, BIT15,  BIT15,       PINMUX_FOR_PM_MSPI_MODE},
    {PAD_PM_GPIO15, PMSLEEP_BASE_ADDR, REG_MSPI_MODE, BIT15,  BIT15,       PINMUX_FOR_PM_MSPI_MODE},
    {PAD_HDMITX_HPD, PMSLEEP_BASE_ADDR, REG_HDMI_HPD_BYPASS, BIT15,  BIT15,       PINMUX_FOR_HDMI_HPD_BYPASS_MODE},
    {PAD_PM_SD30_CDZ, PMSLEEP_BASE_ADDR, REG_SD_CDZ_MODE, BIT14,  BIT14,       PINMUX_FOR_SD_CDZ_MODE},
    {PAD_VID0, PMSLEEP_BASE_ADDR, REG_VID_MODE, BIT5|BIT4,      BIT4,       PINMUX_FOR_VID_MODE},
    {PAD_VID0, PMSLEEP_BASE_ADDR, REG_GPU_VID_MODE, BIT3|BIT2,      BIT2,       PINMUX_FOR_GPU_VID_MODE},
    {PAD_VID0, PMSLEEP_BASE_ADDR, REG_MIIC_MODE, BIT7|BIT6,      BIT7,       PINMUX_FOR_MIIC_MODE},
    {PAD_VID1, PMSLEEP_BASE_ADDR, REG_VID_MODE, BIT5|BIT4,      BIT4,       PINMUX_FOR_VID_MODE},
    {PAD_VID1, PMSLEEP_BASE_ADDR, REG_GPU_VID_MODE, BIT3|BIT2,      BIT2,       PINMUX_FOR_GPU_VID_MODE},
    {PAD_VID1, PMSLEEP_BASE_ADDR, REG_MIIC_MODE, BIT7|BIT6,      BIT7,       PINMUX_FOR_MIIC_MODE},
    {PAD_PM_LED0, PMSLEEP_BASE_ADDR, REG_PM_PWM0_MODE, BIT1|BIT0,      BIT1,       PINMUX_FOR_PM_PWM0_MODE},
    {PAD_PM_LED0, PMSLEEP_BASE_ADDR, REG_LED_MODE, BIT5|BIT4,      BIT4,       PINMUX_FOR_LED_MODE},
    {PAD_PM_LED0, PMSLEEP_BASE_ADDR, REG_PM_PWM1_MODE, BIT1|BIT0,      BIT1,       PINMUX_FOR_PM_PWM1_MODE},
    {PAD_PM_LED1, PMSLEEP_BASE_ADDR, REG_LED_MODE, BIT5|BIT4,      BIT4,       PINMUX_FOR_LED_MODE},
    {PAD_SAR_GPIO0, PM_SAR_BASE_ADDR, REG_SAR_MODE,     BIT0,           BIT0,       PINMUX_FOR_SAR_MODE},
    {PAD_SAR_GPIO1, PM_SAR_BASE_ADDR, REG_SAR_MODE,     BIT1,           BIT1,       PINMUX_FOR_SAR_MODE},
    {PAD_SAR_GPIO2, PM_SAR_BASE_ADDR, REG_SAR_MODE,     BIT2,           BIT2,       PINMUX_FOR_SAR_MODE},
    {PAD_SAR_GPIO3, PM_SAR_BASE_ADDR, REG_SAR_MODE,     BIT3,           BIT3,       PINMUX_FOR_SAR_MODE},
    {PAD_SAR_GPIO4, PM_SAR_BASE_ADDR, REG_SAR_MODE,     BIT4,           BIT4,       PINMUX_FOR_SAR_MODE},
    {PAD_PM_SPI_CZ1, PMSLEEP_BASE_ADDR, REG_SPI_IS_GPIO,     BIT2,           BIT2,       PINMUX_FOR_PMSPI_MODE},
    {PAD_HSYNC_OUT, PADTOP1_BASE_ADDR, REG_THIRDUART_MODE,     BIT8|BIT7|BIT6,           BIT8,       PINMUX_FOR_THIRD_UART_MODE},
    {PAD_HSYNC_OUT, PADTOP1_BASE_ADDR, REG_HSYNC_EN,     BIT6,           BIT6,       PINMUX_FOR_HSYNC_EN},
    {PAD_VSYNC_OUT, PADTOP1_BASE_ADDR, REG_THIRDUART_MODE,     BIT8|BIT7|BIT6,           BIT8,       PINMUX_FOR_THIRD_UART_MODE},
    {PAD_VSYNC_OUT, PADTOP1_BASE_ADDR, REG_VSYNC_EN,     BIT5,           BIT5,       PINMUX_FOR_VSYNC_EN},
    {PAD_BT_I2S_RX_BCK, PADTOP1_BASE_ADDR, REG_MSPI3_MODE1,     BIT2|BIT1|BIT0,           BIT1,       PINMUX_FOR_MSPI3_MODE1},
    {PAD_BT_I2S_RX_BCK, PADTOP1_BASE_ADDR, REG_BT_I2S_TRX_0_MODE,     BIT1,           BIT1,       PINMUX_FOR_BT_I2S_TRX_0_MODE},
    {PAD_BT_I2S_RX_BCK, PADTOP1_BASE_ADDR, REG_BT_I2S_RX_0_MODE,     BIT0,           BIT0,       PINMUX_FOR_BT_I2S_RX_0_MODE},
    {PAD_BT_I2S_RX_BCK, PADTOP1_BASE_ADDR, REG_BT_I2S_TX_0_MODE,     BIT2,           BIT2,       PINMUX_FOR_BT_I2S_TX_0_MODE},
    {PAD_BT_I2S_RX_WCK, PADTOP1_BASE_ADDR, REG_MSPI3_MODE1,     BIT2|BIT1|BIT0,           BIT1,       PINMUX_FOR_MSPI3_MODE1},
    {PAD_BT_I2S_RX_WCK, PADTOP1_BASE_ADDR, REG_BT_I2S_TRX_0_MODE,     BIT1,           BIT1,       PINMUX_FOR_BT_I2S_TRX_0_MODE},
    {PAD_BT_I2S_RX_WCK, PADTOP1_BASE_ADDR, REG_BT_I2S_RX_0_MODE,     BIT0,           BIT0,       PINMUX_FOR_BT_I2S_RX_0_MODE},
    {PAD_BT_I2S_RX_WCK, PADTOP1_BASE_ADDR, REG_BT_I2S_TX_0_MODE,     BIT2,           BIT2,       PINMUX_FOR_BT_I2S_TX_0_MODE},
    {PAD_CODEC_I2S_TX_BCK, PADTOP1_BASE_ADDR, REG_CODEC_I2S_TX_0_MODE,  BIT14, BIT14,       PINMUX_FOR_CODEC_I2S_TX_0_MODE},
    {PAD_CODEC_I2S_TX_WCK, PADTOP1_BASE_ADDR, REG_CODEC_I2S_TX_0_MODE,  BIT14, BIT14,       PINMUX_FOR_CODEC_I2S_TX_0_MODE},
    {PAD_CODEC_I2S_TX_SDO, PADTOP1_BASE_ADDR, REG_CODEC_I2S_TX_0_MODE,  BIT14, BIT14,       PINMUX_FOR_CODEC_I2S_TX_0_MODE},
    {PAD_CODEC_I2S_RX_MCK, PADTOP1_BASE_ADDR, REG_CODEC_I2S_RX_MCK_MODE,  BIT13, BIT13,       PINMUX_FOR_CODEC_I2S_RX_MCK_MODE},
    {PAD_CODEC_I2S_RX_BCK, PADTOP1_BASE_ADDR, REG_CODEC_I2S_RX_0_MODE,  BIT9, BIT9,       PINMUX_FOR_CODEC_I2S_RX_0_MODE},
    {PAD_CODEC_I2S_RX_WCK, PADTOP1_BASE_ADDR, REG_CODEC_I2S_RX_0_MODE,  BIT9, BIT9,       PINMUX_FOR_CODEC_I2S_RX_0_MODE},
    {PAD_CODEC_I2S_RX_SDI0, PADTOP1_BASE_ADDR, REG_CODEC_I2S_RX_0_MODE,  BIT9, BIT9,       PINMUX_FOR_CODEC_I2S_RX_0_MODE},
    {PAD_CODEC_I2S_RX_SDI1, PADTOP1_BASE_ADDR, REG_CODEC_I2S_RX_1_MODE,  BIT10, BIT10,       PINMUX_FOR_CODEC_I2S_RX_1_MODE},
    {PAD_CODEC_I2S_RX_SDI2, PADTOP1_BASE_ADDR, REG_CODEC_I2S_RX_2_MODE,  BIT11, BIT11,       PINMUX_FOR_CODEC_I2S_RX_2_MODE},
    {PAD_CODEC_I2S_RX_SDI3, PADTOP1_BASE_ADDR, REG_CODEC_I2S_RX_3_MODE,  BIT12, BIT12,       PINMUX_FOR_CODEC_I2S_RX_3_MODE},
    {PAD_DMIC_BCK, PADTOP1_BASE_ADDR, REG_DMIC_0_MODE,  BIT1|BIT0, BIT0,       PINMUX_FOR_BT_DMIC_0_MODE},
    {PAD_DMIC_CH0, PADTOP1_BASE_ADDR, REG_DMIC_0_MODE,  BIT1|BIT0, BIT0,       PINMUX_FOR_BT_DMIC_0_MODE},
    {PAD_DMIC_CH1, PADTOP1_BASE_ADDR, REG_DMIC_1_MODE,  BIT3|BIT2, BIT2,       PINMUX_FOR_BT_DMIC_1_MODE},
    {PAD_DMIC_CH2, PADTOP1_BASE_ADDR, REG_DMIC_2_MODE,  BIT5|BIT4, BIT4,       PINMUX_FOR_BT_DMIC_2_MODE},
    {PAD_DMIC_CH3, PADTOP1_BASE_ADDR, REG_DMIC_3_MODE,  BIT7|BIT6, BIT6,       PINMUX_FOR_BT_DMIC_3_MODE},
    {PAD_FUART_TX, PADTOP1_BASE_ADDR, REG_FAST_UART_RTX_MODE,    BIT1|BIT0,      BIT0,  PINMUX_FOR_FAST_UART_RT_MODE},
    {PAD_FUART_RX, PADTOP1_BASE_ADDR, REG_FAST_UART_RTX_MODE,    BIT1|BIT0,      BIT0,  PINMUX_FOR_FAST_UART_RT_MODE},
    {PAD_FUART_RTS, PADTOP1_BASE_ADDR, REG_EJ_CEVA_MODE,    BIT1|BIT0,      BIT0,  PINMUX_FOR_EJ_CEVA_MODE},
    {PAD_FUART_RTS, PADTOP1_BASE_ADDR, REG_EJ_CEVA_MODE,    BIT1|BIT0,      BIT1|BIT0,  PINMUX_FOR_EJ_CEVA_MODE},
    {PAD_FUART_RTS, PADTOP1_BASE_ADDR, REG_FORTHUART_MODE,    BIT2|BIT1|BIT0,      BIT1|BIT0,  PINMUX_FOR_FORTH_UART_MODE},
    {PAD_FUART_RTS, PADTOP1_BASE_ADDR, REG_FUART_MODE,    BIT5|BIT4,      BIT4,  PINMUX_FOR_FUART_MODE},
    {PAD_FUART_CTS, PADTOP1_BASE_ADDR, REG_FORTHUART_MODE,    BIT2|BIT1|BIT0,      BIT1|BIT0,  PINMUX_FOR_FORTH_UART_MODE},
    {PAD_FUART_CTS, PADTOP1_BASE_ADDR, REG_FUART_MODE,    BIT5|BIT4,      BIT4,  PINMUX_FOR_FUART_MODE},
    {PAD_I2C0_SCL, PADTOP1_BASE_ADDR, REG_I2CM0_MODE,    BIT1|BIT0,      BIT0,       PINMUX_FOR_I2CM0_EN},
    {PAD_I2C0_SCL, PADTOP1_BASE_ADDR, REG_I2CM4_MODE,  BIT4|BIT3,      BIT4,       PINMUX_FOR_I2CM4_MODE},
    {PAD_I2C0_SDA, PADTOP1_BASE_ADDR, REG_I2CM0_MODE,    BIT1|BIT0,      BIT0,       PINMUX_FOR_I2CM0_EN},
    {PAD_I2C0_SDA, PADTOP1_BASE_ADDR, REG_I2CM4_MODE,  BIT4|BIT3,      BIT4,       PINMUX_FOR_I2CM4_MODE},
    {PAD_I2C0_SDA, PADTOP1_BASE_ADDR, REG_USB30VCTL_MODE1,  BIT4|BIT3,      BIT4|BIT3,       PINMUX_FOR_USB30VCTL_MODE1},
    {PAD_I2C2_SDA, PADTOP1_BASE_ADDR, REG_I2CM2_MODE,  BIT7|BIT6|BIT5,      BIT5,       PINMUX_FOR_I2CM2_MODE},
    {PAD_I2C2_SDA, PADTOP1_BASE_ADDR, REG_SECONDUART_MODE,  BIT5|BIT4|BIT3,      BIT4,       PINMUX_FOR_SECOND_UART_MODE},
    {PAD_I2C2_SCL, PADTOP1_BASE_ADDR, REG_I2CM2_MODE,  BIT7|BIT6|BIT5,      BIT5,       PINMUX_FOR_I2CM2_MODE},
    {PAD_I2C2_SCL, PADTOP1_BASE_ADDR, REG_SECONDUART_MODE,  BIT5|BIT4|BIT3,      BIT4,       PINMUX_FOR_SECOND_UART_MODE},
    {PAD_I2C3_SDA, PADTOP1_BASE_ADDR, REG_I2CM3_MODE,  BIT10|BIT9|BIT8,      BIT8,       PINMUX_FOR_I2CM3_MODE},
    {PAD_I2C3_SCL, PADTOP1_BASE_ADDR, REG_I2CM3_MODE,  BIT10|BIT9|BIT8,      BIT8,       PINMUX_FOR_I2CM3_MODE},
    {PAD_JTAG_TDO, PADTOP1_BASE_ADDR, REG_EJ_MODE,  BIT2,      BIT2,       PINMUX_FOR_EJ_MODE},
    {PAD_JTAG_TDO, PADTOP1_BASE_ADDR, REG_EJ_CEVA_MODE,  BIT1|BIT0,      BIT1|BIT0,       PINMUX_FOR_EJ_CEVA_MODE},
    {PAD_JTAG_TDI, PADTOP1_BASE_ADDR, REG_EJ_MODE,  BIT2,      BIT2,       PINMUX_FOR_EJ_MODE},
    {PAD_JTAG_TDI, PADTOP1_BASE_ADDR, REG_EJ_CEVA_MODE,  BIT1|BIT0,      BIT1|BIT0,       PINMUX_FOR_EJ_CEVA_MODE},
    {PAD_JTAG_TMS, PADTOP1_BASE_ADDR, REG_EJ_MODE,  BIT2,      BIT2,       PINMUX_FOR_EJ_MODE},
    {PAD_JTAG_TMS, PADTOP1_BASE_ADDR, REG_EJ_CEVA_MODE,  BIT1|BIT0,      BIT1|BIT0,       PINMUX_FOR_EJ_CEVA_MODE},
    {PAD_MIPI_TX_IO0, PADTOP1_BASE_ADDR, REG_PWM0_MODE,  BIT1|BIT0,      BIT1,       PINMUX_FOR_PWM0_MODE},
    {PAD_MIPI_TX_IO0, PADTOP1_BASE_ADDR, REG_MIPI_LVDS_TX_2CH_MODE,  BIT4,      BIT4,       PINMUX_FOR_MIPI_LVDS_TX_2CH_MODE},
    {PAD_MIPI_TX_IO0, PADTOP1_BASE_ADDR, REG_MIPI_LVDS_TX_4CH_MODE,  BIT5,      BIT5,       PINMUX_FOR_MIPI_LVDS_TX_4CH_MODE},
    {PAD_MIPI_TX_IO1, PADTOP1_BASE_ADDR, REG_PWM1_MODE,  BIT3|BIT2,      BIT3,       PINMUX_FOR_PWM1_MODE},
    {PAD_MIPI_TX_IO1, PADTOP1_BASE_ADDR, REG_MIPI_LVDS_TX_2CH_MODE,  BIT4,      BIT4,       PINMUX_FOR_MIPI_LVDS_TX_2CH_MODE},
    {PAD_MIPI_TX_IO1, PADTOP1_BASE_ADDR, REG_MIPI_LVDS_TX_4CH_MODE,  BIT5,      BIT5,       PINMUX_FOR_MIPI_LVDS_TX_4CH_MODE},
    {PAD_MIPI_TX_IO2, PADTOP1_BASE_ADDR, REG_PWM2_MODE,  BIT5|BIT4,      BIT5,       PINMUX_FOR_PWM2_MODE},
    {PAD_MIPI_TX_IO2, PADTOP1_BASE_ADDR, REG_MIPI_LVDS_TX_2CH_MODE,  BIT4,      BIT4,       PINMUX_FOR_MIPI_LVDS_TX_2CH_MODE},
    {PAD_MIPI_TX_IO2, PADTOP1_BASE_ADDR, REG_MIPI_LVDS_TX_4CH_MODE,  BIT5,      BIT5,       PINMUX_FOR_MIPI_LVDS_TX_4CH_MODE},
    {PAD_MIPI_TX_IO3, PADTOP1_BASE_ADDR, REG_PWM3_MODE,  BIT7|BIT6,      BIT7,       PINMUX_FOR_PWM3_MODE},
    {PAD_MIPI_TX_IO3, PADTOP1_BASE_ADDR, REG_MIPI_LVDS_TX_2CH_MODE,  BIT4,      BIT4,       PINMUX_FOR_MIPI_LVDS_TX_2CH_MODE},
    {PAD_MIPI_TX_IO3, PADTOP1_BASE_ADDR, REG_MIPI_LVDS_TX_4CH_MODE,  BIT5,      BIT5,       PINMUX_FOR_MIPI_LVDS_TX_4CH_MODE},
    {PAD_MIPI_TX_IO4, PADTOP1_BASE_ADDR, REG_PWM4_MODE,  BIT9|BIT8,      BIT9,       PINMUX_FOR_PWM4_MODE},
    {PAD_MIPI_TX_IO4, PADTOP1_BASE_ADDR, REG_MIPI_LVDS_TX_2CH_MODE,  BIT4,      BIT4,       PINMUX_FOR_MIPI_LVDS_TX_2CH_MODE},
    {PAD_MIPI_TX_IO4, PADTOP1_BASE_ADDR, REG_MIPI_LVDS_TX_4CH_MODE,  BIT5,      BIT5,       PINMUX_FOR_MIPI_LVDS_TX_4CH_MODE},
    {PAD_MIPI_TX_IO5, PADTOP1_BASE_ADDR, REG_PWM5_MODE,  BIT11|BIT10,      BIT11,       PINMUX_FOR_PWM5_MODE},
    {PAD_MIPI_TX_IO5, PADTOP1_BASE_ADDR, REG_MIPI_LVDS_TX_2CH_MODE,  BIT4,      BIT4,       PINMUX_FOR_MIPI_LVDS_TX_2CH_MODE},
    {PAD_MIPI_TX_IO5, PADTOP1_BASE_ADDR, REG_MIPI_LVDS_TX_4CH_MODE,  BIT5,      BIT5,       PINMUX_FOR_MIPI_LVDS_TX_4CH_MODE},
    {PAD_MIPI_TX_IO6, PADTOP1_BASE_ADDR, REG_MSPI2_MODE1,  BIT7|BIT6,      BIT7,       PINMUX_FOR_MSPI2_MODE1},
    {PAD_MIPI_TX_IO6, PADTOP1_BASE_ADDR, REG_PWM6_MODE,  BIT13|BIT12,      BIT13,       PINMUX_FOR_PWM6_MODE},
    {PAD_MIPI_TX_IO6, PADTOP1_BASE_ADDR, REG_MIPI_LVDS_TX_4CH_MODE,  BIT5,      BIT5,       PINMUX_FOR_MIPI_LVDS_TX_4CH_MODE},
    {PAD_MIPI_TX_IO7, PADTOP1_BASE_ADDR, REG_MSPI2_MODE1,  BIT7|BIT6,      BIT7,       PINMUX_FOR_MSPI2_MODE1},
    {PAD_MIPI_TX_IO7, PADTOP1_BASE_ADDR, REG_PWM7_MODE,  BIT15|BIT14,      BIT15,       PINMUX_FOR_PWM7_MODE},
    {PAD_MIPI_TX_IO7, PADTOP1_BASE_ADDR, REG_MIPI_LVDS_TX_4CH_MODE,  BIT5,      BIT5,       PINMUX_FOR_MIPI_LVDS_TX_4CH_MODE},
    {PAD_MIPI_TX_IO8, PADTOP1_BASE_ADDR, REG_MSPI2_MODE1,  BIT7|BIT6,      BIT7,       PINMUX_FOR_MSPI2_MODE1},
    {PAD_MIPI_TX_IO8, PADTOP1_BASE_ADDR, REG_MIPI_LVDS_TX_4CH_MODE,  BIT5,      BIT5,       PINMUX_FOR_MIPI_LVDS_TX_4CH_MODE},
    {PAD_MIPI_TX_IO9, PADTOP1_BASE_ADDR, REG_MSPI2_MODE1,  BIT7|BIT6,      BIT7,       PINMUX_FOR_MSPI2_MODE1},
    {PAD_MIPI_TX_IO9, PADTOP1_BASE_ADDR, REG_MIPI_LVDS_TX_4CH_MODE,  BIT5,      BIT5,       PINMUX_FOR_MIPI_LVDS_TX_4CH_MODE},
    {PAD_MISC_I2S_TX_BCK, PADTOP1_BASE_ADDR, REG_MISC_I2S_TX_2_MODE,  BIT10,      BIT10,       PINMUX_FOR_MISC_I2S_TX_2_MODE},
    {PAD_MISC_I2S_TX_WCK, PADTOP1_BASE_ADDR, REG_MISC_I2S_TX_2_MODE,  BIT10,      BIT10,       PINMUX_FOR_MISC_I2S_TX_2_MODE},
    {PAD_MISC_I2S_TX_SDO, PADTOP1_BASE_ADDR, REG_MISC_I2S_TX_0_MODE,  BIT8,      BIT8,       PINMUX_FOR_MISC_I2S_TX_0_MODE},
    {PAD_MISC_I2S_TX_SDO, PADTOP1_BASE_ADDR, REG_MISC_I2S_TX_1_MODE,  BIT9,      BIT9,       PINMUX_FOR_MISC_I2S_TX_1_MODE},
    {PAD_MISC_I2S_TX_SDO, PADTOP1_BASE_ADDR, REG_MISC_I2S_TX_2_MODE,  BIT10,      BIT10,       PINMUX_FOR_MISC_I2S_TX_2_MODE},
    {PAD_MISC_I2S_RX_MCK, PADTOP1_BASE_ADDR, REG_SPDIF_IN_MODE,  BIT1|BIT0,      BIT0,       PINMUX_FOR_SPDIF_IN_MODE},
    {PAD_MISC_I2S_RX_MCK, PADTOP1_BASE_ADDR, REG_MISC_I2S_RX_MCK_MODE,  BIT7,      BIT7,       PINMUX_FOR_MISC_I2S_RX_MCK_MODE},
    {PAD_MISC_I2S_RX_BCK, PADTOP1_BASE_ADDR, REG_MISC_I2S_RX_0_MODE,  BIT6,      BIT6,       PINMUX_FOR_MISC_I2S_RX_0_MODE},
    {PAD_MISC_I2S_RX_BCK, PADTOP1_BASE_ADDR, REG_MISC_I2S_TX_0_MODE,  BIT8,      BIT8,       PINMUX_FOR_MISC_I2S_TX_0_MODE},
    {PAD_MISC_I2S_RX_BCK, PADTOP1_BASE_ADDR, REG_DMIC_0_MODE,  BIT1|BIT0,      BIT1,       PINMUX_FOR_BT_DMIC_0_MODE},
    {PAD_MISC_I2S_RX_WCK, PADTOP1_BASE_ADDR, REG_MISC_I2S_RX_0_MODE,  BIT6,      BIT6,       PINMUX_FOR_MISC_I2S_RX_0_MODE},
    {PAD_MISC_I2S_RX_WCK, PADTOP1_BASE_ADDR, REG_MISC_I2S_TX_0_MODE,  BIT8,      BIT8,       PINMUX_FOR_MISC_I2S_TX_0_MODE},
    {PAD_MISC_I2S_RX_WCK, PADTOP1_BASE_ADDR, REG_DMIC_0_MODE,  BIT1|BIT0,      BIT1,       PINMUX_FOR_BT_DMIC_0_MODE},
    {PAD_MISC_I2S_RX_SDI0, PADTOP1_BASE_ADDR, REG_MISC_I2S_RX_0_MODE,  BIT6,      BIT6,       PINMUX_FOR_MISC_I2S_RX_0_MODE},
    {PAD_MISC_I2S_RX_SDI0, PADTOP1_BASE_ADDR, REG_DMIC_1_MODE,  BIT3|BIT2,      BIT3,       PINMUX_FOR_BT_DMIC_1_MODE},
    {PAD_NAND_CEZ0, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT12,       PINMUX_FOR_NAND_MODE},
    {PAD_NAND_CEZ1, PADTOP1_BASE_ADDR, REG_NAND_CS1_EN,  BIT11,      BIT11,       PINMUX_FOR_NAND_CS1_EN},
    {PAD_NAND_ALE, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT12,       PINMUX_FOR_NAND_MODE},
    {PAD_NAND_CLE, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT12,       PINMUX_FOR_NAND_MODE},
    {PAD_NAND_CLE, PADTOP1_BASE_ADDR, REG_SD_CONFIG,  BIT6,      BIT6,       PINMUX_FOR_SD_CONFIG},
    {PAD_NAND_CLE, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT3,       PINMUX_FOR_EMMC_CONFIG},
    {PAD_NAND_WEZ, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT12,       PINMUX_FOR_NAND_MODE},
    {PAD_NAND_WEZ, PADTOP1_BASE_ADDR, REG_SD_CONFIG,  BIT6,      BIT6,       PINMUX_FOR_SD_CONFIG},
    {PAD_NAND_WEZ, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT3,       PINMUX_FOR_EMMC_CONFIG},
    {PAD_NAND_WPZ, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT12,       PINMUX_FOR_NAND_MODE},
    {PAD_NAND_WPZ, PADTOP1_BASE_ADDR, REG_EMMC_RSTN_EN,  BIT6|BIT5,      BIT5,       PINMUX_FOR_EMMC_RSTN_EN},
    {PAD_NAND_WPZ, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT3,       PINMUX_FOR_EMMC_CONFIG},
    {PAD_NAND_REZ, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT12,       PINMUX_FOR_NAND_MODE},
    {PAD_NAND_RBZ, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT12,       PINMUX_FOR_NAND_MODE},
    {PAD_NAND_DA0, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT12,       PINMUX_FOR_NAND_MODE},
    {PAD_NAND_DA0, PADTOP1_BASE_ADDR, REG_SD_CONFIG,  BIT6,      BIT6,       PINMUX_FOR_SD_CONFIG},
    {PAD_NAND_DA0, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT3,       PINMUX_FOR_EMMC_CONFIG},
    {PAD_NAND_DA1, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT12,       PINMUX_FOR_NAND_MODE},
    {PAD_NAND_DA1, PADTOP1_BASE_ADDR, REG_SD_CONFIG,  BIT6,      BIT6,       PINMUX_FOR_SD_CONFIG},
    {PAD_NAND_DA1, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT3,       PINMUX_FOR_EMMC_CONFIG},
    {PAD_NAND_DA2, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT12,       PINMUX_FOR_NAND_MODE},
    {PAD_NAND_DA2, PADTOP1_BASE_ADDR, REG_SD_CONFIG,  BIT6,      BIT6,       PINMUX_FOR_SD_CONFIG},
    {PAD_NAND_DA2, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT3,       PINMUX_FOR_EMMC_CONFIG},
    {PAD_NAND_DA3, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT12,       PINMUX_FOR_NAND_MODE},
    {PAD_NAND_DA3, PADTOP1_BASE_ADDR, REG_SD_CONFIG,  BIT6,      BIT6,       PINMUX_FOR_SD_CONFIG},
    {PAD_NAND_DA3, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT3,       PINMUX_FOR_EMMC_CONFIG},  
    {PAD_NAND_DA4, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT12,       PINMUX_FOR_NAND_MODE},
    {PAD_NAND_DA4, PADTOP1_BASE_ADDR, REG_FAST_UART_RTX_MODE,  BIT1|BIT0,      BIT1,       PINMUX_FOR_FAST_UART_RT_MODE},
    {PAD_NAND_DA4, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT3,       PINMUX_FOR_EMMC_CONFIG},  
    {PAD_NAND_DA5, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT12,       PINMUX_FOR_NAND_MODE},
    {PAD_NAND_DA5, PADTOP1_BASE_ADDR, REG_FAST_UART_RTX_MODE,  BIT1|BIT0,      BIT1,       PINMUX_FOR_FAST_UART_RT_MODE},
    {PAD_NAND_DA5, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT3,       PINMUX_FOR_EMMC_CONFIG},
    {PAD_NAND_DA6, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT12,       PINMUX_FOR_NAND_MODE},
    {PAD_NAND_DA6, PADTOP1_BASE_ADDR, REG_I2CM2_MODE,  BIT7|BIT6|BIT5,      BIT7,       PINMUX_FOR_I2CM2_MODE},
    {PAD_NAND_DA6, PADTOP1_BASE_ADDR, REG_FUART_MODE,  BIT5|BIT4,      BIT5,       PINMUX_FOR_FUART_MODE},
    {PAD_NAND_DA6, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT3,       PINMUX_FOR_EMMC_CONFIG},
    {PAD_NAND_DA7, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT12,       PINMUX_FOR_NAND_MODE},
    {PAD_NAND_DA7, PADTOP1_BASE_ADDR, REG_I2CM2_MODE,  BIT7|BIT6|BIT5,      BIT7,       PINMUX_FOR_I2CM2_MODE},
    {PAD_NAND_DA7, PADTOP1_BASE_ADDR, REG_FUART_MODE,  BIT5|BIT4,      BIT5,       PINMUX_FOR_FUART_MODE},
    {PAD_NAND_DA7, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT3,       PINMUX_FOR_EMMC_CONFIG},

    {PAD_RGMII_0_MDIO, PADTOP1_BASE_ADDR, REG_GT0_MDIO,  BIT6,      BIT6,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_TX_CTL, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_TX_CTL, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT8,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_MDC, PADTOP1_BASE_ADDR, REG_GT0_MDIO,  BIT6,      BIT6,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_TXD3, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_RX_CLK, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_RX_CLK, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT8,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_TXD2, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_RXD3, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_TXD1, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_TXD1, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT8,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_RXD2, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_TXD0, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_TXD0, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT8,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_RXD1, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_RXD1, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT8,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_TX_CLK, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_RXD0, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_RXD0, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT8,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_RX_CTL, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_0_RX_CTL, PADTOP1_BASE_ADDR, REG_GT0_MODE,  BIT8|BIT7,      BIT8,       PINMUX_FOR_GT0_MODE},
    {PAD_RGMII_1_TX_CTL, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT9,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_TX_CTL, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT10,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_RX_CLK, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT9,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_RX_CLK, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT10,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_TXD3, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT9,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_RXD3, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT9,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_TXD2, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT9,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_RXD2, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT9,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_TXD1, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT9,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_TXD1, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT10,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_RXD1, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT9,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_RXD1, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT10,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_TXD0, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT9,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_TXD0, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT10,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_RXD0, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT9,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_RXD0, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT10,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_TX_CLK, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT9,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_RX_CTL, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,     BIT9,       PINMUX_FOR_GT1_MODE},
    {PAD_RGMII_1_RX_CTL, PADTOP1_BASE_ADDR, REG_GT1_MODE,  BIT10|BIT9,      BIT10,       PINMUX_FOR_GT1_MODE},

    {PAD_PWM0, PADTOP1_BASE_ADDR, REG_PWM0_MODE,  BIT1|BIT0,      BIT0,       PINMUX_FOR_PWM0_MODE},
    {PAD_PWM1, PADTOP1_BASE_ADDR, REG_PWM1_MODE,  BIT1|BIT0,      BIT0,       PINMUX_FOR_PWM1_MODE},
    {PAD_SD_CLK, PADTOP1_BASE_ADDR, REG_SDIO_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_SDIO_MODE},
    {PAD_SD_CMD, PADTOP1_BASE_ADDR, REG_SDIO_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_SDIO_MODE},
    {PAD_SD_D0, PADTOP1_BASE_ADDR, REG_SDIO_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_SDIO_MODE},
    {PAD_SD_D1, PADTOP1_BASE_ADDR, REG_SDIO_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_SDIO_MODE},
    {PAD_SD_D2, PADTOP1_BASE_ADDR, REG_SDIO_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_SDIO_MODE},
    {PAD_SD_D3, PADTOP1_BASE_ADDR, REG_SDIO_MODE,  BIT8|BIT7,      BIT7,       PINMUX_FOR_SDIO_MODE},

    {PAD_SD30_IO0, PADTOP1_BASE_ADDR, REG_SDIO30_MODE,  BIT7,      BIT7,       PINMUX_FOR_SDIO_MODE},
    {PAD_SD30_IO0, PADTOP1_BASE_ADDR, REG_DMIC_0_MODE,  BIT1|BIT0,      BIT1|BIT0,       PINMUX_FOR_DMIC_MODE},
    {PAD_SD30_IO1, PADTOP1_BASE_ADDR, REG_SDIO30_MODE,  BIT7,      BIT7,       PINMUX_FOR_SDIO_MODE},
    {PAD_SD30_IO1, PADTOP1_BASE_ADDR, REG_DMIC_0_MODE,  BIT1|BIT0,      BIT1|BIT0,       PINMUX_FOR_DMIC_MODE},
    {PAD_SD30_IO2, PADTOP1_BASE_ADDR, REG_SDIO30_MODE,  BIT7,      BIT7,       PINMUX_FOR_SDIO_MODE},
    {PAD_SD30_IO2, PADTOP1_BASE_ADDR, REG_DMIC_1_MODE,  BIT3|BIT2,      BIT3|BIT2,       PINMUX_FOR_DMIC_MODE},
    {PAD_SD30_IO3, PADTOP1_BASE_ADDR, REG_SDIO30_MODE,  BIT7,      BIT7,       PINMUX_FOR_SDIO_MODE},
    {PAD_SD30_IO3, PADTOP1_BASE_ADDR, REG_DMIC_2_MODE,  BIT5|BIT4,      BIT5,       PINMUX_FOR_DMIC_MODE},
    {PAD_SD30_IO4, PADTOP1_BASE_ADDR, REG_SDIO30_MODE,  BIT7,      BIT7,       PINMUX_FOR_SDIO_MODE},
    {PAD_SD30_IO4, PADTOP1_BASE_ADDR, REG_DMIC_2_MODE,  BIT7|BIT6,      BIT7,       PINMUX_FOR_DMIC_MODE},
    {PAD_SD30_IO5, PADTOP1_BASE_ADDR, REG_SDIO30_MODE,  BIT7,      BIT7,       PINMUX_FOR_SDIO_MODE},

    {PAD_SNR0_D0, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT13,       PINMUX_FOR_NAND_MODE},
    {PAD_SNR0_D0, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT4,       PINMUX_FOR_EMMC_MODE},
    {PAD_SNR0_D0, PADTOP1_BASE_ADDR, REG_CCIR0_8B_MODE,  BIT4,      BIT4,       PINMUX_FOR_CCIR0_8B_MODE},
    {PAD_SNR0_D0, PADTOP1_BASE_ADDR, REG_CCIR0_16B_MODE,  BIT3,      BIT3,       PINMUX_FOR_CCIR0_16B_MODE},
    {PAD_SNR0_D1, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT13,       PINMUX_FOR_NAND_MODE},
    {PAD_SNR0_D1, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT4,       PINMUX_FOR_EMMC_MODE},
    {PAD_SNR0_D1, PADTOP1_BASE_ADDR, REG_CCIR0_8B_MODE,  BIT4,      BIT4,       PINMUX_FOR_CCIR0_8B_MODE},
    {PAD_SNR0_D1, PADTOP1_BASE_ADDR, REG_CCIR0_16B_MODE,  BIT3,      BIT3,       PINMUX_FOR_CCIR0_16B_MODE},
    {PAD_SNR0_D2, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT13,       PINMUX_FOR_NAND_MODE},
    {PAD_SNR0_D2, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT4,       PINMUX_FOR_EMMC_MODE},
    {PAD_SNR0_D2, PADTOP1_BASE_ADDR, REG_CCIR0_8B_MODE,  BIT4,      BIT4,       PINMUX_FOR_CCIR0_8B_MODE},
    {PAD_SNR0_D2, PADTOP1_BASE_ADDR, REG_CCIR0_16B_MODE,  BIT3,      BIT3,       PINMUX_FOR_CCIR0_16B_MODE},
    {PAD_SNR0_D3, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT13,       PINMUX_FOR_NAND_MODE},
    {PAD_SNR0_D3, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT4,       PINMUX_FOR_EMMC_MODE},
    {PAD_SNR0_D3, PADTOP1_BASE_ADDR, REG_CCIR0_8B_MODE,  BIT4,      BIT4,       PINMUX_FOR_CCIR0_8B_MODE},
    {PAD_SNR0_D4, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT13,       PINMUX_FOR_NAND_MODE},
    {PAD_SNR0_D4, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT4,       PINMUX_FOR_EMMC_MODE},
    {PAD_SNR0_D4, PADTOP1_BASE_ADDR, REG_CCIR0_CLK_MODE,  BIT7|BIT6|BIT5,      BIT5,       PINMUX_FOR_CCIR0_CLK_MODE},
    {PAD_SNR0_D5, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT13,       PINMUX_FOR_NAND_MODE},
    {PAD_SNR0_D5, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT4,       PINMUX_FOR_EMMC_MODE},
    {PAD_SNR0_D6, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT13,       PINMUX_FOR_NAND_MODE},
    {PAD_SNR0_D6, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT4,       PINMUX_FOR_EMMC_MODE},
    {PAD_SNR0_D6, PADTOP1_BASE_ADDR, REG_CCIR0_8B_MODE,  BIT4,      BIT4,       PINMUX_FOR_CCIR0_8B_MODE},
    {PAD_SNR0_D6, PADTOP1_BASE_ADDR, REG_CCIR0_16B_MODE,  BIT3,      BIT3,       PINMUX_FOR_CCIR0_16B_MODE},

    {PAD_SNR0_D7, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT13,       PINMUX_FOR_NAND_MODE},
    {PAD_SNR0_D7, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT4,       PINMUX_FOR_EMMC_MODE},
    {PAD_SNR0_D7, PADTOP1_BASE_ADDR, REG_SNR0_IN_MODE,  BIT11,      BIT11,       PINMUX_FOR_SNR0_MODE},
    {PAD_SNR0_D7, PADTOP1_BASE_ADDR, REG_CCIR0_8B_MODE,  BIT4,      BIT4,       PINMUX_FOR_CCIR0_8B_MODE},
    {PAD_SNR0_D7, PADTOP1_BASE_ADDR, REG_CCIR0_16B_MODE,  BIT3,      BIT3,       PINMUX_FOR_CCIR0_16B_MODE},
    {PAD_SNR0_D8, PADTOP1_BASE_ADDR, REG_SNR0_IN_MODE,  BIT11,      BIT11,       PINMUX_FOR_SNR0_MODE},    
    {PAD_SNR0_D8, PADTOP1_BASE_ADDR, REG_CCIR0_8B_MODE,  BIT4,      BIT4,       PINMUX_FOR_CCIR0_8B_MODE},
    {PAD_SNR0_D8, PADTOP1_BASE_ADDR, REG_CCIR0_16B_MODE,  BIT3,      BIT3,       PINMUX_FOR_CCIR0_16B_MODE},
    {PAD_SNR0_D9, PADTOP1_BASE_ADDR, REG_SNR0_IN_MODE,  BIT11,      BIT11,       PINMUX_FOR_SNR0_MODE},    
    {PAD_SNR0_D9, PADTOP1_BASE_ADDR, REG_CCIR0_8B_MODE,  BIT4,      BIT4,       PINMUX_FOR_CCIR0_8B_MODE},
    {PAD_SNR0_D9, PADTOP1_BASE_ADDR, REG_CCIR0_16B_MODE,  BIT3,      BIT3,       PINMUX_FOR_CCIR0_16B_MODE},
    {PAD_SNR0_GPIO0, PADTOP1_BASE_ADDR, REG_SNR0_IN_MODE,  BIT11,      BIT11,       PINMUX_FOR_SNR0_MODE},    
    {PAD_SNR0_GPIO1, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT13,       PINMUX_FOR_NAND_MODE},
    {PAD_SNR0_GPIO1, PADTOP1_BASE_ADDR, REG_SNR0_IN_MODE,  BIT11,      BIT11,       PINMUX_FOR_SNR0_MODE},
    {PAD_SNR0_GPIO2, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT13,       PINMUX_FOR_NAND_MODE},
    {PAD_SNR0_GPIO2, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT4,       PINMUX_FOR_EMMC_MODE},
    {PAD_SNR0_GPIO2, PADTOP1_BASE_ADDR, REG_SNR0_IN_MODE,  BIT11,      BIT11,       PINMUX_FOR_SNR0_MODE},
    {PAD_SNR0_GPIO2, PADTOP1_BASE_ADDR, REG_CCIR0_CLK_MODE,  BIT7|BIT6|BIT5,      BIT6,       PINMUX_FOR_CCIR0_CLK_MODE},
    {PAD_SNR0_GPIO3, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT13,       PINMUX_FOR_NAND_MODE},
    {PAD_SNR0_GPIO3, PADTOP1_BASE_ADDR, REG_CCIR0_CTRL_MODE,  BIT9|BIT8,      BIT8,       PINMUX_FOR_CCIR0_CTRL_MODE},
    {PAD_SNR0_GPIO4, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT13,       PINMUX_FOR_NAND_MODE},
    {PAD_SNR0_GPIO4, PADTOP1_BASE_ADDR, REG_EMMC_RSTN_EN,  BIT14|BIT13,      BIT14,       PINMUX_FOR_EMMC_RSTN_EN},
    {PAD_SNR0_GPIO4, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT4,       PINMUX_FOR_EMMC_CONFIG},
    {PAD_SNR0_GPIO4, PADTOP1_BASE_ADDR, REG_CCIR0_CTRL_MODE,  BIT9|BIT8,      BIT8,       PINMUX_FOR_CCIR0_CTRL_MODE},
    {PAD_SNR0_GPIO5, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT13,       PINMUX_FOR_NAND_MODE},
    {PAD_SNR0_GPIO5, PADTOP1_BASE_ADDR, REG_EMMC_CONFIG,  BIT4|BIT3,      BIT4,       PINMUX_FOR_EMMC_CONFIG},
    {PAD_SNR0_GPIO5, PADTOP1_BASE_ADDR, REG_NAND_MODE,  BIT13|BIT12,      BIT13,       PINMUX_FOR_NAND_MODE},
    {PAD_SNR0_GPIO6, PADTOP1_BASE_ADDR, REG_I2CM1_MODE,  BIT4|BIT3|BIT2,      BIT2,       PINMUX_FOR_I2C1_MODE},
    {PAD_SNR0_GPIO6, PADTOP1_BASE_ADDR, REG_I2CM1_MODE,  BIT4|BIT3|BIT2,      BIT3|BIT2,       PINMUX_FOR_I2C1_MODE},
    {PAD_SNR0_GPIO6, PADTOP1_BASE_ADDR, REG_I2CM1_MODE,  BIT4|BIT3|BIT2,      BIT4,       PINMUX_FOR_I2C1_MODE},
    {PAD_SNR0_GPIO7, PADTOP1_BASE_ADDR, REG_I2CM1_MODE,  BIT4|BIT3|BIT2,      BIT2,       PINMUX_FOR_I2C1_MODE},
    {PAD_SNR0_GPIO7, PADTOP1_BASE_ADDR, REG_I2CM1_MODE,  BIT4|BIT3|BIT2,      BIT3|BIT2,       PINMUX_FOR_I2C1_MODE},
    {PAD_SNR0_GPIO7, PADTOP1_BASE_ADDR, REG_I2CM1_MODE,  BIT4|BIT3|BIT2,      BIT4,       PINMUX_FOR_I2C1_MODE},

    {PAD_SPI0_CZ0, PADTOP1_BASE_ADDR, REG_EJ_CEVA_MODE,  BIT1|BIT0,      BIT0,       PINMUX_FOR_EJ_CEVA_MODE},
    {PAD_SPI0_CZ0, PADTOP1_BASE_ADDR, REG_I2CM4_MODE,  BIT12|BIT11,      BIT11,       PINMUX_FOR_I2CM4_MODE},
    {PAD_SPI0_CZ0, PADTOP1_BASE_ADDR, REG_THIRDUART_MODE,  BIT8|BIT7|BIT6,      BIT7,       PINMUX_FOR_THIRD_UART_MODE},
    {PAD_SPI0_CZ0, PADTOP1_BASE_ADDR, REG_MSPI1_MODE1,  BIT1|BIT0,      BIT0,        PINMUX_FOR_MSPI1_MODE1},
    {PAD_SPI0_CK, PADTOP1_BASE_ADDR, REG_EJ_CEVA_MODE,  BIT1|BIT0,      BIT0,       PINMUX_FOR_EJ_CEVA_MODE},
    {PAD_SPI0_CK, PADTOP1_BASE_ADDR, REG_THIRDUART_MODE,  BIT8|BIT7|BIT6,      BIT7,       PINMUX_FOR_THIRD_UART_MODE},
    {PAD_SPI0_CK, PADTOP1_BASE_ADDR, REG_MSPI1_MODE1,  BIT1|BIT0,      BIT0,        PINMUX_FOR_MSPI1_MODE1},
    {PAD_SPI0_CK, PADTOP1_BASE_ADDR, REG_MSPI1_MODE1,  BIT1|BIT1,      BIT0,        PINMUX_FOR_MSPI1_MODE1},
    {PAD_SPI0_DO, PADTOP1_BASE_ADDR, REG_EJ_CEVA_MODE,  BIT1|BIT0,      BIT0,       PINMUX_FOR_EJ_CEVA_MODE},
    {PAD_SPI0_DO, PADTOP1_BASE_ADDR, REG_I2CM4_MODE,  BIT12|BIT11,      BIT11,       PINMUX_FOR_I2CM4_MODE},
    {PAD_SPI0_DO, PADTOP1_BASE_ADDR, REG_FORTHUART_MODE,  BIT2|BIT1|BIT0,      BIT2,       PINMUX_FOR_FORTH_UART_MODE},
    {PAD_SPI0_DO, PADTOP1_BASE_ADDR, REG_MSPI1_MODE1,  BIT1|BIT0,      BIT0,        PINMUX_FOR_MSPI1_MODE1},
    {PAD_SPI0_DO, PADTOP1_BASE_ADDR, REG_MSPI1_MODE1,  BIT1|BIT0,      BIT1,        PINMUX_FOR_MSPI1_MODE1},
    {PAD_SPI0_DI, PADTOP1_BASE_ADDR, REG_EJ_CEVA_MODE,  BIT1|BIT0,      BIT0,       PINMUX_FOR_EJ_CEVA_MODE},
    {PAD_SPI0_DI, PADTOP1_BASE_ADDR, REG_FORTHUART_MODE,  BIT2|BIT1|BIT0,      BIT2,       PINMUX_FOR_FORTH_UART_MODE},
    {PAD_SPI0_DI, PADTOP1_BASE_ADDR, REG_MSPI1_MODE1,  BIT1|BIT0,      BIT0,        PINMUX_FOR_MSPI1_MODE1},
    {PAD_SPI0_DI, PADTOP1_BASE_ADDR, REG_MSPI1_MODE1,  BIT1|BIT0,      BIT1,        PINMUX_FOR_MSPI1_MODE1},
    {PAD_SPI1_CK, PADTOP1_BASE_ADDR, REG_MSPI2_MODE1,  BIT7|BIT6,      BIT6,        PINMUX_FOR_MSPI2_MODE1},
    {PAD_SPI1_CZ0, PADTOP1_BASE_ADDR, REG_MSPI2_MODE1,  BIT7|BIT6,      BIT6,        PINMUX_FOR_MSPI2_MODE1},
    {PAD_SPI1_DO, PADTOP1_BASE_ADDR, REG_MSPI2_MODE1,  BIT7|BIT6,      BIT6,        PINMUX_FOR_MSPI2_MODE1},
    {PAD_SPI1_DI, PADTOP1_BASE_ADDR, REG_MSPI2_MODE1,  BIT7|BIT6,      BIT6,        PINMUX_FOR_MSPI2_MODE1},

    {PAD_SNR1_GPIO0, PADTOP1_BASE_ADDR, REG_SNR1_IN_MODE,  BIT12,      BIT12,        PINMUX_FOR_SNR1_IN_MODE},
    {PAD_SNR1_GPIO1, PADTOP1_BASE_ADDR, REG_SNR1_IN_MODE,  BIT12,      BIT12,        PINMUX_FOR_SNR1_IN_MODE},

    
    {PAD_PM_SPI_CK, PMSLEEP_BASE_ADDR, REG_SPI_IS_GPIO,  BIT0,     BIT0,        PINMUX_FOR_GPIO_MODE},
    {PAD_PM_SPI_DI, PMSLEEP_BASE_ADDR, REG_SPI_IS_GPIO,  BIT0,      BIT0,        PINMUX_FOR_GPIO_MODE},
    {PAD_PM_SPI_DO, PMSLEEP_BASE_ADDR, REG_SPI_IS_GPIO, BIT0,      BIT0,        PINMUX_FOR_GPIO_MODE},
    {PAD_PM_SPI_CZ, PMSLEEP_BASE_ADDR, REG_SPI_IS_GPIO,  BIT1,      BIT1,       PINMUX_FOR_GPIO_MODE},
    {PAD_PM_SPI_HOLDZ, PMSLEEP_BASE_ADDR, REG_SPI_IS_GPIO,BIT7|BIT6,      BIT7|BIT6,       PINMUX_FOR_SPI1_MODE},
    {PAD_PM_SPI_WPZ, PMSLEEP_BASE_ADDR, REG_SPI_IS_GPIO, BIT4,     BIT4,       PINMUX_FOR_PMSPI_MODE},

};

/*TBD*/
#if 0
    {PAD_FUART_RX, PADTOP1_BASE_ADDR, REG_EJ_MODE,      BIT1|BIT0,      BIT1,       PINMUX_FOR_EJ_MODE},
    {PAD_FUART_RX, PADTOP1_BASE_ADDR, REG_SPI0_MODE,    BIT1|BIT0,      BIT1|BIT0,  PINMUX_FOR_SPI0_MODE}, /* SPI0 mode3 */
    {PAD_FUART_RX, PADTOP1_BASE_ADDR, REG_FUART_MODE,   BIT1|BIT0,      BIT0,       PINMUX_FOR_FUART_MODE}, /* FUART mode1 */
    {PAD_FUART_RX, PADTOP1_BASE_ADDR, REG_UART0_MODE,   BIT5|BIT4,      BIT5,       PINMUX_FOR_UART0_MODE}, /* UART0 mode2 */
    {PAD_FUART_RX, PADTOP1_BASE_ADDR, REG_PWM0_MODE,    BIT1|BIT0,      BIT1|BIT0,  PINMUX_FOR_PWM0_MODE}, /* PWM0 mode3 */

    {PAD_FUART_TX, PADTOP1_BASE_ADDR, REG_EJ_MODE,      BIT1|BIT0,      BIT1,       PINMUX_FOR_EJ_MODE},
    {PAD_FUART_TX, PADTOP1_BASE_ADDR, REG_SPI0_MODE,    BIT1|BIT0,      BIT1|BIT0,  PINMUX_FOR_SPI0_MODE}, /* SPI0 mode3 */
    {PAD_FUART_TX, PADTOP1_BASE_ADDR, REG_FUART_MODE,   BIT1|BIT0,      BIT0,       PINMUX_FOR_FUART_MODE}, /* FUART mode1 */
    {PAD_FUART_TX, PADTOP1_BASE_ADDR, REG_UART0_MODE,   BIT5|BIT4,      BIT5,       PINMUX_FOR_UART0_MODE}, /* UART0 mode2 */
    {PAD_FUART_TX, PADTOP1_BASE_ADDR, REG_PWM1_MODE,    BIT3|BIT2,      BIT3|BIT2,  PINMUX_FOR_PWM1_MODE}, /* PWM1 mode3 */

    {PAD_FUART_CTS, PADTOP1_BASE_ADDR, REG_EJ_MODE,     BIT1|BIT0,      BIT1,       PINMUX_FOR_EJ_MODE},
    {PAD_FUART_CTS, PADTOP1_BASE_ADDR, REG_SPI0_MODE,   BIT1|BIT0,      BIT1|BIT0,  PINMUX_FOR_SPI0_MODE}, /* SPI0 mode3 */
    {PAD_FUART_CTS, PADTOP1_BASE_ADDR, REG_FUART_MODE,  BIT1|BIT0,      BIT0,       PINMUX_FOR_FUART_MODE}, /* FUART mode1 */
    {PAD_FUART_CTS, PADTOP1_BASE_ADDR, REG_UART1_MODE,  BIT9|BIT8,      BIT9,       PINMUX_FOR_UART1_MODE}, /* UART1 mode2 */
    {PAD_FUART_CTS, PADTOP1_BASE_ADDR, REG_PWM2_MODE,   BIT5|BIT4,      BIT5,       PINMUX_FOR_PWM2_MODE}, /* PWM2 mode2 */

    {PAD_FUART_RTS, PADTOP1_BASE_ADDR, REG_EJ_MODE,     BIT1|BIT0,      BIT1,       PINMUX_FOR_EJ_MODE},
    {PAD_FUART_RTS, PADTOP1_BASE_ADDR, REG_SPI0_MODE,   BIT1|BIT0,      BIT1|BIT0,  PINMUX_FOR_SPI0_MODE}, /* SPI0 mode3 */
    {PAD_FUART_RTS, PADTOP1_BASE_ADDR, REG_FUART_MODE,  BIT1|BIT0,      BIT0,       PINMUX_FOR_FUART_MODE}, /* FUART mode1 */
    {PAD_FUART_RTS, PADTOP1_BASE_ADDR, REG_UART1_MODE,  BIT9|BIT8,      BIT9,       PINMUX_FOR_UART1_MODE}, /* UART1 mode2 */
    {PAD_FUART_RTS, PADTOP1_BASE_ADDR, REG_PWM3_MODE,   BIT7|BIT6,      BIT7,       PINMUX_FOR_PWM3_MODE}, /* PWM3 mode2 */

    {PAD_I2C0_SCL, PADTOP1_BASE_ADDR, REG_I2C0_MODE,    BIT1|BIT0,      BIT0,       PINMUX_FOR_I2C0_MODE}, /* I2C0 mode1 */
    {PAD_I2C0_SCL, PADTOP1_BASE_ADDR, REG_SR_I2C_MODE,  BIT5|BIT4,      BIT4,       PINMUX_FOR_SR_MODE}, /* SR mode1 */

    {PAD_I2C0_SDA, PADTOP1_BASE_ADDR, REG_I2C0_MODE,    BIT1|BIT0,      BIT0,       PINMUX_FOR_I2C0_MODE}, /* I2C0 mode1 */
    {PAD_I2C0_SDA, PADTOP1_BASE_ADDR, REG_SR_I2C_MODE,  BIT5|BIT4,      BIT4,       PINMUX_FOR_SR_MODE}, /* SR mode1 */

    {PAD_I2C1_SCL, PADTOP1_BASE_ADDR, REG_I2C1_MODE,    BIT5|BIT4,      BIT4,       PINMUX_FOR_I2C1_MODE}, /* I2C1 mode1 */
    {PAD_I2C1_SCL, PADTOP1_BASE_ADDR, REG_SR_I2C_MODE,  BIT5|BIT4,      BIT5,       PINMUX_FOR_SR_MODE}, /* SR mode2 */

    {PAD_I2C1_SDA, PADTOP1_BASE_ADDR, REG_I2C1_MODE,    BIT5|BIT4,      BIT4,       PINMUX_FOR_I2C1_MODE}, /* I2C1 mode1 */
    {PAD_I2C1_SDA, PADTOP1_BASE_ADDR, REG_SR_I2C_MODE,  BIT5|BIT4,      BIT5,       PINMUX_FOR_SR_MODE}, /* SR mode2 */

    {PAD_SR_IO00, PADTOP1_BASE_ADDR, REG_I2C0_MODE,     BIT1|BIT0,      BIT1|BIT0,  PINMUX_FOR_I2C0_MODE}, /* I2C0 mode3 */
    {PAD_SR_IO00, PADTOP1_BASE_ADDR, REG_I2C1_MODE,     BIT5|BIT4,      BIT5|BIT4,  PINMUX_FOR_I2C1_MODE}, /* I2C1 mode3 */
    {PAD_SR_IO00, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2,       PINMUX_FOR_SR_MODE}, /* SR mode4 */
    {PAD_SR_IO00, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO00, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */

    {PAD_SR_IO01, PADTOP1_BASE_ADDR, REG_I2C0_MODE,     BIT1|BIT0,      BIT1|BIT0,  PINMUX_FOR_I2C0_MODE}, /* I2C0 mode3 */
    {PAD_SR_IO01, PADTOP1_BASE_ADDR, REG_I2C1_MODE,     BIT5|BIT4,      BIT5|BIT4,  PINMUX_FOR_I2C1_MODE}, /* I2C1 mode3 */
    {PAD_SR_IO01, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2,       PINMUX_FOR_SR_MODE}, /* SR mode4 */
    {PAD_SR_IO01, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO01, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */

    {PAD_SR_IO02, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT0,       PINMUX_FOR_SR_MODE}, /* SR mode1 */
    {PAD_SR_IO02, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1,       PINMUX_FOR_SR_MODE}, /* SR mode2 */
    {PAD_SR_IO02, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode3 */
    {PAD_SR_IO02, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2,       PINMUX_FOR_SR_MODE}, /* SR mode4 */
    {PAD_SR_IO02, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO02, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */
    {PAD_SR_IO02, PADTOP1_BASE_ADDR, REG_CCIR_MODE,     BIT4,           BIT4,       PINMUX_FOR_CCIR_MODE}, /* CCIR mode */

    {PAD_SR_IO03, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT0,       PINMUX_FOR_SR_MODE}, /* SR mode1 */
    {PAD_SR_IO03, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1,       PINMUX_FOR_SR_MODE}, /* SR mode2 */
    {PAD_SR_IO03, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode3 */
    {PAD_SR_IO03, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2,       PINMUX_FOR_SR_MODE}, /* SR mode4 */
    {PAD_SR_IO03, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO03, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */
    {PAD_SR_IO03, PADTOP1_BASE_ADDR, REG_CCIR_MODE,     BIT4,           BIT4,       PINMUX_FOR_CCIR_MODE}, /* CCIR mode */

    {PAD_SR_IO04, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT0,       PINMUX_FOR_SR_MODE}, /* SR mode1 */
    {PAD_SR_IO04, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1,       PINMUX_FOR_SR_MODE}, /* SR mode2 */
    {PAD_SR_IO04, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode3 */
    {PAD_SR_IO04, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2,       PINMUX_FOR_SR_MODE}, /* SR mode4 */
    {PAD_SR_IO04, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO04, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */
    {PAD_SR_IO04, PADTOP1_BASE_ADDR, REG_CCIR_MODE,     BIT4,           BIT4,       PINMUX_FOR_CCIR_MODE}, /* CCIR mode */

    {PAD_SR_IO05, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT0,       PINMUX_FOR_SR_MODE}, /* SR mode1 */
    {PAD_SR_IO05, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1,       PINMUX_FOR_SR_MODE}, /* SR mode2 */
    {PAD_SR_IO05, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode3 */
    {PAD_SR_IO05, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2,       PINMUX_FOR_SR_MODE}, /* SR mode4 */
    {PAD_SR_IO05, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO05, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */
    {PAD_SR_IO05, PADTOP1_BASE_ADDR, REG_CCIR_MODE,     BIT4,           BIT4,       PINMUX_FOR_CCIR_MODE}, /* CCIR mode */

    {PAD_SR_IO06, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT0,       PINMUX_FOR_SR_MODE}, /* SR mode1 */
    {PAD_SR_IO06, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1,       PINMUX_FOR_SR_MODE}, /* SR mode2 */
    {PAD_SR_IO06, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode3 */
    {PAD_SR_IO06, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2,       PINMUX_FOR_SR_MODE}, /* SR mode4 */
    {PAD_SR_IO06, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO06, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */
    {PAD_SR_IO06, PADTOP1_BASE_ADDR, REG_CCIR_MODE,     BIT4,           BIT4,       PINMUX_FOR_CCIR_MODE}, /* CCIR mode */

    {PAD_SR_IO07, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT0,       PINMUX_FOR_SR_MODE}, /* SR mode1 */
    {PAD_SR_IO07, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1,       PINMUX_FOR_SR_MODE}, /* SR mode2 */
    {PAD_SR_IO07, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode3 */
    {PAD_SR_IO07, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2,       PINMUX_FOR_SR_MODE}, /* SR mode4 */
    {PAD_SR_IO07, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO07, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */
    {PAD_SR_IO07, PADTOP1_BASE_ADDR, REG_CCIR_MODE,     BIT4,           BIT4,       PINMUX_FOR_CCIR_MODE}, /* CCIR mode */

    {PAD_SR_IO08, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT0,       PINMUX_FOR_SR_MODE}, /* SR mode1 */
    {PAD_SR_IO08, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1,       PINMUX_FOR_SR_MODE}, /* SR mode2 */
    {PAD_SR_IO08, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode3 */
    {PAD_SR_IO08, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2,       PINMUX_FOR_SR_MODE}, /* SR mode4 */
    {PAD_SR_IO08, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO08, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */
    {PAD_SR_IO08, PADTOP1_BASE_ADDR, REG_CCIR_MODE,     BIT4,           BIT4,       PINMUX_FOR_CCIR_MODE}, /* CCIR mode */

    {PAD_SR_IO09, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT0,       PINMUX_FOR_SR_MODE}, /* SR mode1 */
    {PAD_SR_IO09, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1,       PINMUX_FOR_SR_MODE}, /* SR mode2 */
    {PAD_SR_IO09, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode3 */
    {PAD_SR_IO09, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2,       PINMUX_FOR_SR_MODE}, /* SR mode4 */
    {PAD_SR_IO09, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO09, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */
    {PAD_SR_IO09, PADTOP1_BASE_ADDR, REG_CCIR_MODE,     BIT4,           BIT4,       PINMUX_FOR_CCIR_MODE}, /* CCIR mode */

    {PAD_SR_IO10, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT0,       PINMUX_FOR_SR_MODE}, /* SR mode1 */
    {PAD_SR_IO10, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1,       PINMUX_FOR_SR_MODE}, /* SR mode2 */
    {PAD_SR_IO10, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode3 */
    {PAD_SR_IO10, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2,       PINMUX_FOR_SR_MODE}, /* SR mode4 */
    {PAD_SR_IO10, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO10, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */
    {PAD_SR_IO10, PADTOP1_BASE_ADDR, REG_CCIR_MODE,     BIT4,           BIT4,       PINMUX_FOR_CCIR_MODE}, /* CCIR mode */

    {PAD_SR_IO11, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT0,       PINMUX_FOR_SR_MODE}, /* SR mode1 */
    {PAD_SR_IO11, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1,       PINMUX_FOR_SR_MODE}, /* SR mode2 */
    {PAD_SR_IO11, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode3 */
    {PAD_SR_IO11, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2,       PINMUX_FOR_SR_MODE}, /* SR mode4 */
    {PAD_SR_IO11, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO11, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */
    {PAD_SR_IO11, PADTOP1_BASE_ADDR, REG_CCIR_MODE,     BIT4,           BIT4,       PINMUX_FOR_CCIR_MODE}, /* CCIR mode */

    {PAD_SR_IO12, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT0,       PINMUX_FOR_SR_MODE}, /* SR mode1 */
    {PAD_SR_IO12, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1,       PINMUX_FOR_SR_MODE}, /* SR mode2 */
    {PAD_SR_IO12, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode3 */
    {PAD_SR_IO12, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2,       PINMUX_FOR_SR_MODE}, /* SR mode4 */
    {PAD_SR_IO12, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO12, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */
    {PAD_SR_IO12, PADTOP1_BASE_ADDR, REG_CCIR_MODE,     BIT4,           BIT4,       PINMUX_FOR_CCIR_MODE}, /* CCIR mode */

    {PAD_SR_IO13, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT0,       PINMUX_FOR_SR_MODE}, /* SR mode1 */
    {PAD_SR_IO13, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1,       PINMUX_FOR_SR_MODE}, /* SR mode2 */
    {PAD_SR_IO13, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode3 */
    {PAD_SR_IO13, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2,       PINMUX_FOR_SR_MODE}, /* SR mode4 */
    {PAD_SR_IO13, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO13, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */

    {PAD_SR_IO14, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT0,       PINMUX_FOR_SR_MODE}, /* SR mode1 */
    {PAD_SR_IO14, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1,       PINMUX_FOR_SR_MODE}, /* SR mode2 */
    {PAD_SR_IO14, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode3 */
    {PAD_SR_IO14, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2,       PINMUX_FOR_SR_MODE}, /* SR mode4 */
    {PAD_SR_IO14, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO14, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */

    {PAD_SR_IO15, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT0,       PINMUX_FOR_SR_MODE}, /* SR mode1 */
    {PAD_SR_IO15, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1,       PINMUX_FOR_SR_MODE}, /* SR mode2 */
    {PAD_SR_IO15, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode3 */
    {PAD_SR_IO15, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2,       PINMUX_FOR_SR_MODE}, /* SR mode4 */
    {PAD_SR_IO15, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO15, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */

    {PAD_SR_IO16, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT0,       PINMUX_FOR_SR_MODE}, /* SR mode1 */
    {PAD_SR_IO16, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1,       PINMUX_FOR_SR_MODE}, /* SR mode2 */
    {PAD_SR_IO16, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode3 */
    {PAD_SR_IO16, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO16, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */

    {PAD_SR_IO17, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT0,       PINMUX_FOR_SR_MODE}, /* SR mode1 */
    {PAD_SR_IO17, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1,       PINMUX_FOR_SR_MODE}, /* SR mode2 */
    {PAD_SR_IO17, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT1|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode3 */
    {PAD_SR_IO17, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT0,  PINMUX_FOR_SR_MODE}, /* SR mode5 */
    {PAD_SR_IO17, PADTOP1_BASE_ADDR, REG_SR_MODE,       BIT2|BIT1|BIT0, BIT2|BIT1,  PINMUX_FOR_SR_MODE}, /* SR mode6 */

    {PAD_NAND_ALE, PADTOP1_BASE_ADDR, REG_NAND_MODE,    BIT0,           BIT0,   PINMUX_FOR_NAND_MODE}, /* Nand mode1 */
    {PAD_NAND_ALE, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,   PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_NAND_CLE, PADTOP1_BASE_ADDR, REG_NAND_MODE,    BIT0,           BIT0,   PINMUX_FOR_NAND_MODE}, /* Nand mode1 */
    {PAD_NAND_CLE, PADTOP1_BASE_ADDR, REG_SD_MODE,      BIT3|BIT2,      BIT2,   PINMUX_FOR_SD_MODE}, /* SD mode1 */
    {PAD_NAND_CLE, PADTOP1_BASE_ADDR, REG_EMMC_MODE,    BIT0,           BIT0,   PINMUX_FOR_EMMC_MODE},  /* EMMC mode1 */
    {PAD_NAND_CLE, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,   PINMUX_FOR_TTL_MODE},  /* TTL mode1 */

    {PAD_NAND_CEZ, PADTOP1_BASE_ADDR, REG_NAND_MODE,    BIT0,           BIT0,   PINMUX_FOR_NAND_MODE}, /* Nand mode1 */
    {PAD_NAND_CEZ, PADTOP1_BASE_ADDR, REG_SD_MODE,      BIT3|BIT2,      BIT2,   PINMUX_FOR_SD_MODE}, /* SD mode1 */
    {PAD_NAND_CEZ, PADTOP1_BASE_ADDR, REG_EMMC_MODE,    BIT0,           BIT0,   PINMUX_FOR_EMMC_MODE},  /* EMMC mode1 */
    {PAD_NAND_CEZ, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,   PINMUX_FOR_TTL_MODE},  /* TTL mode1 */

    {PAD_NAND_WEZ, PADTOP1_BASE_ADDR, REG_NAND_MODE,    BIT0,           BIT0,   PINMUX_FOR_NAND_MODE}, /* Nand mode1 */
    {PAD_NAND_WEZ, PADTOP1_BASE_ADDR, REG_SD_MODE,      BIT3|BIT2,      BIT2,   PINMUX_FOR_SD_MODE}, /* SD mode1 */
    {PAD_NAND_WEZ, PADTOP1_BASE_ADDR, REG_EMMC_MODE,    BIT0,           BIT0,   PINMUX_FOR_EMMC_MODE},  /* EMMC mode1 */
    {PAD_NAND_WEZ, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,   PINMUX_FOR_TTL_MODE},  /* TTL mode1 */

    {PAD_NAND_WPZ, PADTOP1_BASE_ADDR, REG_NAND_MODE,    BIT0,           BIT0,   PINMUX_FOR_NAND_MODE}, /* Nand mode1 */
    {PAD_NAND_WPZ, PADTOP1_BASE_ADDR, REG_SD_MODE,      BIT3|BIT2,      BIT2,   PINMUX_FOR_SD_MODE}, /* SD mode1 */
    {PAD_NAND_WPZ, PADTOP1_BASE_ADDR, REG_EMMC_MODE,    BIT0,           BIT0,   PINMUX_FOR_EMMC_MODE},  /* EMMC mode1 */
    {PAD_NAND_WPZ, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,   PINMUX_FOR_TTL_MODE},  /* TTL mode1 */

    {PAD_NAND_REZ, PADTOP1_BASE_ADDR, REG_NAND_MODE,    BIT0,           BIT0,   PINMUX_FOR_NAND_MODE}, /* Nand mode1 */
    {PAD_NAND_REZ, PADTOP1_BASE_ADDR, REG_SD_MODE,      BIT3|BIT2,      BIT2,   PINMUX_FOR_SD_MODE}, /* SD mode1 */
    {PAD_NAND_REZ, PADTOP1_BASE_ADDR, REG_EMMC_MODE,    BIT0,           BIT0,   PINMUX_FOR_EMMC_MODE},  /* EMMC mode1 */
    {PAD_NAND_REZ, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,   PINMUX_FOR_TTL_MODE},  /* TTL mode1 */

    {PAD_NAND_RBZ, PADTOP1_BASE_ADDR, REG_NAND_MODE,    BIT0,           BIT0,   PINMUX_FOR_NAND_MODE}, /* Nand mode1 */
    {PAD_NAND_RBZ, PADTOP1_BASE_ADDR, REG_SD_MODE,      BIT3|BIT2,      BIT2,   PINMUX_FOR_SD_MODE}, /* SD mode1 */
    {PAD_NAND_RBZ, PADTOP1_BASE_ADDR, REG_EMMC_MODE,    BIT0,           BIT0,   PINMUX_FOR_EMMC_MODE},  /* EMMC mode1 */
    {PAD_NAND_RBZ, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,   PINMUX_FOR_TTL_MODE},  /* TTL mode1 */

    {PAD_NAND_DA0, PADTOP1_BASE_ADDR, REG_NAND_MODE,    BIT0,           BIT0,   PINMUX_FOR_NAND_MODE}, /* Nand mode1 */
    {PAD_NAND_DA0, PADTOP1_BASE_ADDR, REG_EMMC_MODE,    BIT1,           BIT1,   PINMUX_FOR_EMMC_MODE},  /* EMMC mode1 */
    {PAD_NAND_DA0, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,   PINMUX_FOR_TTL_MODE},  /* TTL mode1 */

    {PAD_NAND_DA1, PADTOP1_BASE_ADDR, REG_NAND_MODE,    BIT0,           BIT0,   PINMUX_FOR_NAND_MODE}, /* Nand mode1 */
    {PAD_NAND_DA1, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,   PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_NAND_DA2, PADTOP1_BASE_ADDR, REG_NAND_MODE,    BIT0,           BIT0,       PINMUX_FOR_NAND_MODE}, /* Nand mode1 */
    {PAD_NAND_DA2, PADTOP1_BASE_ADDR, REG_PWM2_MODE,    BIT5|BIT4,      BIT5|BIT4,  PINMUX_FOR_PWM2_MODE}, /* PWM2 mode3 */
    {PAD_NAND_DA2, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_NAND_DA3, PADTOP1_BASE_ADDR, REG_NAND_MODE,    BIT0,           BIT0,       PINMUX_FOR_NAND_MODE}, /* Nand mode1 */
    {PAD_NAND_DA3, PADTOP1_BASE_ADDR, REG_PWM3_MODE,    BIT7|BIT6,      BIT7|BIT6,  PINMUX_FOR_PWM3_MODE}, /* PWM3 mode3 */
    {PAD_NAND_DA3, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_NAND_DA4, PADTOP1_BASE_ADDR, REG_NAND_MODE,    BIT0,           BIT0,       PINMUX_FOR_NAND_MODE}, /* Nand mode1 */
    {PAD_NAND_DA4, PADTOP1_BASE_ADDR, REG_EMMC_MODE,    BIT0,           BIT0,       PINMUX_FOR_EMMC_MODE},  /* EMMC mode1 */
    {PAD_NAND_DA4, PADTOP1_BASE_ADDR, REG_PWM4_MODE,    BIT9|BIT8,      BIT8,       PINMUX_FOR_PWM4_MODE}, /* PWM4 mode1 */
    {PAD_NAND_DA4, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_NAND_DA5, PADTOP1_BASE_ADDR, REG_NAND_MODE,    BIT0,           BIT0,       PINMUX_FOR_NAND_MODE}, /* Nand mode1 */
    {PAD_NAND_DA5, PADTOP1_BASE_ADDR, REG_EMMC_MODE,    BIT0,           BIT0,       PINMUX_FOR_EMMC_MODE},  /* EMMC mode1 */
    {PAD_NAND_DA5, PADTOP1_BASE_ADDR, REG_PWM5_MODE,    BIT11|BIT10,    BIT10,       PINMUX_FOR_PWM5_MODE}, /* PWM5 mode1 */
    {PAD_NAND_DA5, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_NAND_DA6, PADTOP1_BASE_ADDR, REG_NAND_MODE,    BIT0,           BIT0,       PINMUX_FOR_NAND_MODE}, /* Nand mode1 */
    {PAD_NAND_DA6, PADTOP1_BASE_ADDR, REG_EMMC_MODE,    BIT0,           BIT0,       PINMUX_FOR_EMMC_MODE},  /* EMMC mode1 */
    {PAD_NAND_DA6, PADTOP1_BASE_ADDR, REG_PWM6_MODE,    BIT13|BIT12,    BIT12,      PINMUX_FOR_PWM6_MODE}, /* PWM6 mode1 */
    {PAD_NAND_DA6, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_NAND_DA7, PADTOP1_BASE_ADDR, REG_NAND_MODE,    BIT0,           BIT0,       PINMUX_FOR_NAND_MODE}, /* Nand mode1 */
    {PAD_NAND_DA7, PADTOP1_BASE_ADDR, REG_EMMC_MODE,    BIT0,           BIT0,       PINMUX_FOR_EMMC_MODE},  /* EMMC mode1 */
    {PAD_NAND_DA7, PADTOP1_BASE_ADDR, REG_PWM7_MODE,    BIT15|BIT14,    BIT14,      PINMUX_FOR_PWM7_MODE}, /* PWM7 mode1 */
    {PAD_NAND_DA7, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_UART0_RX, PADTOP1_BASE_ADDR, REG_UART0_MODE,   BIT5|BIT4,      BIT4,       PINMUX_FOR_UART0_MODE}, /* UART0 mode1 */
    {PAD_UART0_RX, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_UART0_TX, PADTOP1_BASE_ADDR, REG_UART0_MODE,   BIT5|BIT4,      BIT4,       PINMUX_FOR_UART0_MODE}, /* UART0 mode1 */
    {PAD_UART0_TX, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_UART1_RX, PADTOP1_BASE_ADDR, REG_UART1_MODE,   BIT9|BIT8,      BIT8,       PINMUX_FOR_UART1_MODE},/* UART1 mode1 */
    {PAD_UART1_RX, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_UART1_TX, PADTOP1_BASE_ADDR, REG_UART1_MODE,   BIT9|BIT8,      BIT8,       PINMUX_FOR_UART1_MODE},/* UART1 mode1 */
    {PAD_UART1_TX, PADTOP1_BASE_ADDR, REG_TTL_MODE,     BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_SPI0_CZ, PADTOP1_BASE_ADDR, REG_EJ_MODE,       BIT1|BIT0,      BIT1,       PINMUX_FOR_EJ_MODE}, /* EJ mode2 */
    {PAD_SPI0_CZ, PADTOP1_BASE_ADDR, REG_SPI0_MODE,     BIT1|BIT0,      BIT0,       PINMUX_FOR_SPI0_MODE}, /* SPI0 mode1 */
    {PAD_SPI0_CZ, PADTOP1_BASE_ADDR, REG_PWM4_MODE,     BIT9|BIT8,      BIT9,       PINMUX_FOR_PWM4_MODE}, /* PWM4 mode2 */
    {PAD_SPI0_CZ, PADTOP1_BASE_ADDR, REG_TTL_MODE,      BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_SPI0_CK, PADTOP1_BASE_ADDR, REG_EJ_MODE,       BIT1|BIT0,      BIT1,       PINMUX_FOR_EJ_MODE}, /* EJ mode2 */
    {PAD_SPI0_CK, PADTOP1_BASE_ADDR, REG_SPI0_MODE,     BIT1|BIT0,      BIT0,       PINMUX_FOR_SPI0_MODE}, /* SPI0 mode1 */
    {PAD_SPI0_CK, PADTOP1_BASE_ADDR, REG_PWM5_MODE,     BIT11|BIT10,    BIT11,      PINMUX_FOR_PWM5_MODE}, /* PWM5 mode2 */
    {PAD_SPI0_CK, PADTOP1_BASE_ADDR, REG_TTL_MODE,      BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_SPI0_DI, PADTOP1_BASE_ADDR, REG_EJ_MODE,       BIT1|BIT0,      BIT1,       PINMUX_FOR_EJ_MODE}, /* EJ mode2 */
    {PAD_SPI0_DI, PADTOP1_BASE_ADDR, REG_SPI0_MODE,     BIT1|BIT0,      BIT0,       PINMUX_FOR_SPI0_MODE}, /* SPI0 mode1 */
    {PAD_SPI0_DI, PADTOP1_BASE_ADDR, REG_PWM6_MODE,     BIT13|BIT12,    BIT13,      PINMUX_FOR_PWM6_MODE}, /* PWM6 mode2 */
    {PAD_SPI0_DI, PADTOP1_BASE_ADDR, REG_TTL_MODE,      BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_SPI0_DO, PADTOP1_BASE_ADDR, REG_EJ_MODE,       BIT1|BIT0,      BIT1,       PINMUX_FOR_EJ_MODE}, /* EJ mode2 */
    {PAD_SPI0_DO, PADTOP1_BASE_ADDR, REG_SPI0_MODE,     BIT1|BIT0,      BIT0,       PINMUX_FOR_SPI0_MODE}, /* SPI0 mode1 */
    {PAD_SPI0_DO, PADTOP1_BASE_ADDR, REG_PWM7_MODE,     BIT15|BIT14,    BIT15,      PINMUX_FOR_PWM7_MODE}, /* PWM7 mode2 */
    {PAD_SPI0_DO, PADTOP1_BASE_ADDR, REG_TTL_MODE,      BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_SPI1_CZ, PADTOP1_BASE_ADDR, REG_SPI1_MODE,     BIT6|BIT5|BIT4, BIT4,       PINMUX_FOR_SPI1_MODE}, /* SPI1 mode1 */
    {PAD_SPI1_CZ, PADTOP1_BASE_ADDR, REG_TTL_MODE,      BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_SPI1_CK, PADTOP1_BASE_ADDR, REG_SPI1_MODE,     BIT6|BIT5|BIT4, BIT4,       PINMUX_FOR_SPI1_MODE}, /* SPI1 mode1 */
    {PAD_SPI1_CK, PADTOP1_BASE_ADDR, REG_TTL_MODE,      BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_SPI1_DI, PADTOP1_BASE_ADDR, REG_SPI1_MODE,     BIT6|BIT5|BIT4, BIT4,       PINMUX_FOR_SPI1_MODE}, /* SPI1 mode1 */
    {PAD_SPI1_DI, PADTOP1_BASE_ADDR, REG_TTL_MODE,      BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_SPI1_DO, PADTOP1_BASE_ADDR, REG_SPI1_MODE,     BIT6|BIT5|BIT4, BIT4,       PINMUX_FOR_SPI1_MODE}, /* SPI1 mode1 */
    {PAD_SPI1_DO, PADTOP1_BASE_ADDR, REG_TTL_MODE,      BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_PWM0, PADTOP1_BASE_ADDR, REG_I2C0_MODE,        BIT1|BIT0,      BIT1,       PINMUX_FOR_I2C0_MODE}, /* I2C0 mode2 */
    {PAD_PWM0, PADTOP1_BASE_ADDR, REG_I2C1_MODE,        BIT5|BIT4,      BIT5,       PINMUX_FOR_I2C1_MODE}, /* I2C1 mode2 */
    {PAD_PWM0, PADTOP1_BASE_ADDR, REG_PWM0_MODE,        BIT1|BIT0,      BIT0,       PINMUX_FOR_PWM0_MODE}, /* PWM0 mode1 */
    {PAD_PWM0, PADTOP1_BASE_ADDR, REG_TTL_MODE,         BIT6,           BIT6,       PINMUX_FOR_TTL_MODE}, /* TTL mode1 */

    {PAD_PWM1, PADTOP1_BASE_ADDR, REG_I2C0_MODE,        BIT1|BIT0,      BIT1,       PINMUX_FOR_I2C0_MODE}, /* I2C0 mode2 */
    {PAD_PWM1, PADTOP1_BASE_ADDR, REG_I2C1_MODE,        BIT5|BIT4,      BIT5,       PINMUX_FOR_I2C1_MODE}, /* I2C1 mode2 */
    {PAD_PWM1, PADTOP1_BASE_ADDR, REG_PWM1_MODE,        BIT3|BIT2,      BIT2,       PINMUX_FOR_PWM1_MODE}, /* PWM1 mode1 */

    {PAD_SD_CLK, PADTOP1_BASE_ADDR, REG_SDIO_MODE,      BIT8,           BIT8,       PINMUX_FOR_SDIO_MODE}, /* SDIO mode1 */
    {PAD_SD_CLK, PADTOP1_BASE_ADDR, REG_SD_MODE,        BIT3|BIT2,      BIT3,       PINMUX_FOR_SD_MODE}, /* SD mode2 */

    {PAD_SD_CMD, PADTOP1_BASE_ADDR, REG_SDIO_MODE,      BIT8,           BIT8,       PINMUX_FOR_SDIO_MODE}, /* SDIO mode1 */
    {PAD_SD_CMD, PADTOP1_BASE_ADDR, REG_SD_MODE,        BIT3|BIT2,      BIT3,       PINMUX_FOR_SD_MODE}, /* SD mode2 */

    {PAD_SD_D0, PADTOP1_BASE_ADDR, REG_SPI1_MODE,       BIT6|BIT5|BIT4, BIT5|BIT4,  PINMUX_FOR_SPI1_MODE}, /* SPI1 mode3 */
    {PAD_SD_D0, PADTOP1_BASE_ADDR, REG_SDIO_MODE,       BIT8,           BIT8,       PINMUX_FOR_SDIO_MODE}, /* SDIO mode1 */
    {PAD_SD_D0, PADTOP1_BASE_ADDR, REG_SD_MODE,         BIT3|BIT2,      BIT3,       PINMUX_FOR_SD_MODE}, /* SD mode2 */

    {PAD_SD_D1, PADTOP1_BASE_ADDR, REG_SPI1_MODE,       BIT6|BIT5|BIT4, BIT5|BIT4,  PINMUX_FOR_SPI1_MODE}, /* SPI1 mode3 */
    {PAD_SD_D1, PADTOP1_BASE_ADDR, REG_SDIO_MODE,       BIT8,           BIT8,       PINMUX_FOR_SDIO_MODE}, /* SDIO mode1 */
    {PAD_SD_D1, PADTOP1_BASE_ADDR, REG_SD_MODE,         BIT3|BIT2,      BIT3,       PINMUX_FOR_SD_MODE}, /* SD mode2 */

    {PAD_SD_D2, PADTOP1_BASE_ADDR, REG_SPI1_MODE,       BIT6|BIT5|BIT4, BIT5|BIT4,  PINMUX_FOR_SPI1_MODE}, /* SPI1 mode3 */
    {PAD_SD_D2, PADTOP1_BASE_ADDR, REG_SDIO_MODE,       BIT8,           BIT8,       PINMUX_FOR_SDIO_MODE}, /* SDIO mode1 */
    {PAD_SD_D2, PADTOP1_BASE_ADDR, REG_SD_MODE,         BIT3|BIT2,      BIT3,       PINMUX_FOR_SD_MODE}, /* SD mode2 */

    {PAD_SD_D3, PADTOP1_BASE_ADDR, REG_SPI1_MODE,       BIT6|BIT5|BIT4, BIT5|BIT4,  PINMUX_FOR_SPI1_MODE}, /* SPI1 mode3 */
    {PAD_SD_D3, PADTOP1_BASE_ADDR, REG_SDIO_MODE,       BIT8,           BIT8,       PINMUX_FOR_SDIO_MODE}, /* SDIO mode1 */
    {PAD_SD_D3, PADTOP1_BASE_ADDR, REG_SD_MODE,         BIT3|BIT2,      BIT3,       PINMUX_FOR_SD_MODE}, /* SD mode2 */

    {PAD_PM_SD_CDZ, PMSLEEP_BASE_ADDR, REG_SD_CDZ_MODE, BIT14,          BIT14,      PINMUX_FOR_SD_MODE},

    /* PAD_PM_IRIN special case*/

    {PAD_PM_SPI_CZ, PMSLEEP_BASE_ADDR, REG_SPI_IS_GPIO, BIT3,           BIT3,       PINMUX_FOR_PMSPI_MODE},

    {PAD_PM_SPI_CK, PMSLEEP_BASE_ADDR, REG_SPI_IS_GPIO, BIT3,           BIT3,       PINMUX_FOR_PMSPI_MODE},

    {PAD_PM_SPI_DI, PMSLEEP_BASE_ADDR, REG_SPI_IS_GPIO, BIT3,           BIT3,       PINMUX_FOR_PMSPI_MODE},

    {PAD_PM_SPI_DO, PMSLEEP_BASE_ADDR, REG_SPI_IS_GPIO, BIT3,           BIT3,       PINMUX_FOR_PMSPI_MODE},

    {PAD_PM_SPI_WPZ, PMSLEEP_BASE_ADDR, REG_SPI_IS_GPIO,BIT3,           BIT3,       PINMUX_FOR_PMSPI_MODE},

    {PAD_PM_SPI_HLD, PMSLEEP_BASE_ADDR, REG_SPI_IS_GPIO,BIT7|BIT6,      BIT6,       PINMUX_FOR_SPI1_MODE},

    {PAD_PM_LED0, PMSLEEP_BASE_ADDR, REG_LED_MODE,      BIT5|BIT4,      BIT4,       PINMUX_FOR_LED_MODE},

    {PAD_PM_LED1, PMSLEEP_BASE_ADDR, REG_LED_MODE,      BIT5|BIT4,      BIT4,       PINMUX_FOR_LED_MODE},

    {PAD_SAR_GPIO0, PM_SAR_BASE_ADDR, REG_SAR_MODE,     BIT0,           BIT0,       PINMUX_FOR_SAR_MODE},

    {PAD_SAR_GPIO1, PM_SAR_BASE_ADDR, REG_SAR_MODE,     BIT1,           BIT1,       PINMUX_FOR_SAR_MODE},

    {PAD_SAR_GPIO2, PM_SAR_BASE_ADDR, REG_SAR_MODE,     BIT2,           BIT2,       PINMUX_FOR_SAR_MODE},

    {PAD_SAR_GPIO3, PM_SAR_BASE_ADDR, REG_SAR_MODE,     BIT3,           BIT3,       PINMUX_FOR_SAR_MODE},

    {PAD_ETH_RN, ALBANY2_BASE_ADDR, REG_ETH_GPIO_EN,    BIT0,           BIT0,       PINMUX_FOR_ETH_MODE},

    {PAD_ETH_RP, ALBANY2_BASE_ADDR, REG_ETH_GPIO_EN,    BIT1,           BIT1,       PINMUX_FOR_ETH_MODE},

    {PAD_ETH_TN, ALBANY2_BASE_ADDR, REG_ETH_GPIO_EN,    BIT2,           BIT2,       PINMUX_FOR_ETH_MODE},

    {PAD_ETH_TP, ALBANY2_BASE_ADDR, REG_ETH_GPIO_EN,    BIT3,           BIT3,       PINMUX_FOR_ETH_MODE},
};
#endif

S32 halCheckPin(U32 padID)
{
    if (GPIO_NR <= padID)
        return FALSE;

    return TRUE;
}

S32 halPadGetVal(U32 padID, U32* mode)
{
    U8 i = 0;
    U8 fgModeIsFind = 0;

    if (FALSE == halCheckPin(padID))
    {
        return FALSE;
    }

    *mode = PINMUX_FOR_GPIO_MODE;

    switch(padID)
    {
    /* PAD_PM_GPIO4 special case */
    case PAD_PM_GPIO4:
        if( INREG16((PMSLEEP_BASE_ADDR*0x200) + (REG_GPIO_PM_LOCK*0x4)) == 0xBABE )
        {
            return TRUE;
        }
        else
        {
            *mode = PINMUX_FOR_UNKNOWN_MODE;
            return FALSE;
        }

    case PAD_PM_SPI_HOLDZ:
         if(INREGMSK16( PMSLEEP_BASE_ADDR*0x200 + REG_SPI_IS_GPIO*4, BIT6|BIT7) == 0x0)
        {
            return TRUE;
        }
        else
        {
            *mode = PINMUX_FOR_UNKNOWN_MODE;
            return FALSE;
        }

    case PAD_PM_SPI_WPZ:
         if(INREGMSK16( PMSLEEP_BASE_ADDR*0x200 + REG_SPI_IS_GPIO*4, BIT4) == 0x0)
        {
            return TRUE;
        }
        else
        {
            *mode = PINMUX_FOR_UNKNOWN_MODE;
            return FALSE;
        }

    case PAD_GPIO0:
    case PAD_GPIO1:
    case PAD_GPIO2:
    case PAD_GPIO3:
    case PAD_PM_GPIO2:
    case PAD_PM_GPIO5:
    case PAD_PM_GPIO6:
    case PAD_PM_GPIO7:
    case PAD_PM_GPIO9:
    case PAD_PM_GPIO10:
    case PAD_PM_GPIO11:
        return TRUE;

    default:
        for(i=0; i<sizeof(padmux_table)/sizeof(struct stPadmux); i++)
        {
            if(padmux_table[i].padID == padID )
            {
                if( (INREG16( ((padmux_table[i].base)*0x200) + ((padmux_table[i].offset)*0x4))&(padmux_table[i].mask)) == padmux_table[i].val )
                {
                    *mode =  padmux_table[i].mode;
                    fgModeIsFind = 1;
                    break;
                }

            }
        }
    }


    if(!fgModeIsFind)
    {
        return FALSE;
    }

    return TRUE;
}


S32 halPadSetVal(U32 padID, U32 mode)
{
    U16 U16Reg_val = -1;
    U8 fgModeIsFind = 0;
        U16 i;

    if (FALSE == halCheckPin(padID))
    {
        return FALSE;
    }

    switch(padID)
    {

    case PAD_PM_GPIO4:
        if(mode == PINMUX_FOR_GPIO_MODE)
        {
            OUTREG16( PMSLEEP_BASE_ADDR*0x200 + REG_GPIO_PM_LOCK*4, 0xBABE);
        }
        return TRUE;
    case PAD_PM_SPI_HOLDZ:
         if(mode == PINMUX_FOR_GPIO_MODE)
        {
            OUTREGMSK16( PMSLEEP_BASE_ADDR*0x200 + REG_SPI_IS_GPIO*4, 0x0, BIT6|BIT7);
        }
        return TRUE;

    case PAD_PM_SPI_WPZ:
         if(mode == PINMUX_FOR_GPIO_MODE)
        {
            OUTREGMSK16( PMSLEEP_BASE_ADDR*0x200 + REG_SPI_IS_GPIO*4, 0x0, BIT4);
        }
        return TRUE;

    case PAD_GPIO0:
    case PAD_GPIO1:
    case PAD_GPIO2:
    case PAD_GPIO3:
    case PAD_PM_GPIO2:        
    case PAD_PM_GPIO5:
    case PAD_PM_GPIO6:
    case PAD_PM_GPIO7:
    case PAD_PM_GPIO9:
    case PAD_PM_GPIO10:
    case PAD_PM_GPIO11:
        return TRUE;

    default:
        for(i=0; i<sizeof(padmux_table)/sizeof(struct stPadmux); i++)
        {
            if(padID == padmux_table[i].padID)
            {
                if(mode == padmux_table[i].mode)
                {
                    U16Reg_val = INREG16( (padmux_table[i].base)*0x200 + (padmux_table[i].offset)*0x4 );
                    U16Reg_val &= ~(padmux_table[i].mask);
                    U16Reg_val |= padmux_table[i].val;

                    OUTREG16( (padmux_table[i].base)*0x200 + (padmux_table[padID+i].offset)*4, U16Reg_val);
                    fgModeIsFind = 1;

                    break;
                }
                else
                {
                    //Clear high priority setting
                    U16Reg_val = INREG16( (padmux_table[i].base)*0x200 + (padmux_table[i].offset)*0x4 );
                    if((U16Reg_val&padmux_table[i].mask) == padmux_table[i].val)
                    {
                        U16Reg_val &= (~padmux_table[i].mask);
                        OUTREG16( (padmux_table[i].base)*0x200 + (padmux_table[i].offset)*0x4, U16Reg_val);
                    }
                }
            }
        }
    }
    if(!fgModeIsFind)
    {
        return FALSE;
    }
    return TRUE;
}


