/****************************************************************************
*
* FILENAME:        $RCSfile: toml.c,v $
*
* DESCRIPTION:     toml parser
*
* vi: set ts=4:
*
* Copyright (c) 2018 by Grandstream Networks, Inc.
* All rights reserved.
* 
* This material is proprietary to Grandstream Networks, Inc. and, 
* in addition to the above mentioned Copyright, may be 
* subject to protection under other intellectual property 
* regimes, including patents, trade secrets, designs and/or 
* trademarks. 
*
* Any use of this material for any purpose, except with an 
* express license from Grandstream Networks, Inc. is strictly 
* prohibited. 
*
***************************************************************************/

//=================
//  Includes
//=================

//=================
// Defines
//=================
#ifndef __KERNEL__
#if 0
#define ARCH_32BIT
#else
#define ARCH_64BIT
#endif

#define MAX_ERRNO 4095
#ifdef ARCH_32BIT
#define IS_ERR_VALUE(x) ((x) >= (unsigned int)-MAX_ERRNO)
#else
#define IS_ERR_VALUE(x) ((x) >= (unsigned long)-MAX_ERRNO)
#endif
#endif

//=================
//  Globals
//=================

//=================
// Typedefs
//=================
typedef enum {
    TOML_TYPE_ROOT,
    TOML_TYPE_KV,
    TOML_TYPE_TABLE,
} toml_type_t;

struct toml_object;

struct toml_kv {
    struct toml_kv *next;

    char *key;
    int key_len;
    char *value;
    int value_len;
};

struct toml_table {
	char *tag;
    int tag_len;

    struct toml_object *container;
};

struct toml_object {
    struct toml_object *next;
    struct toml_object *parent;
    struct toml_object *children;
    toml_type_t type;
    int array;

    union {
        struct toml_kv toml_kv;
        struct toml_table toml_table;
    } u;
};

//=================
// Functions
//=================
#ifndef __KERNEL__
#ifdef ARCH_32BIT
void *ERR_PTR(int error);
#else
void *ERR_PTR(long error);
#endif

#ifdef ARCH_32BIT
int PTR_ERR(const void *ptr);
#else
long PTR_ERR(const void *ptr);
#endif

#ifdef ARCH_32BIT
int IS_ERR(const void *ptr);
#else
long IS_ERR(const void *ptr);
#endif
#endif

struct toml_object *toml_init( void );
int toml_parse( struct toml_object *root, char *buffer );
void toml_dump( struct toml_object *root );
void toml_free( struct toml_object *root );
